from ...tl.mtproto_request import MTProtoRequest


class UpdateChatAdmins(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateChatAdmins#6e947941 chat_id:int enabled:Bool version:int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x6e947941

    def __init__(self, chat_id, enabled, version):
        """
        :param chat_id: Telegram type: «int».
        :param enabled: Telegram type: «Bool».
        :param version: Telegram type: «int».

        Constructor for Update: Instance of either UpdateNewMessage, UpdateMessageID, UpdateDeleteMessages, UpdateUserTyping, UpdateChatUserTyping, UpdateChatParticipants, UpdateUserStatus, UpdateUserName, UpdateUserPhoto, UpdateContactRegistered, UpdateContactLink, UpdateNewEncryptedMessage, UpdateEncryptedChatTyping, UpdateEncryption, UpdateEncryptedMessagesRead, UpdateChatParticipantAdd, UpdateChatParticipantDelete, UpdateDcOptions, UpdateUserBlocked, UpdateNotifySettings, UpdateServiceNotification, UpdatePrivacy, UpdateUserPhone, UpdateReadHistoryInbox, UpdateReadHistoryOutbox, UpdateWebPage, UpdateReadMessagesContents, UpdateChannelTooLong, UpdateChannel, UpdateNewChannelMessage, UpdateReadChannelInbox, UpdateDeleteChannelMessages, UpdateChannelMessageViews, UpdateChatAdmins, UpdateChatParticipantAdmin, UpdateNewStickerSet, UpdateStickerSetsOrder, UpdateStickerSets, UpdateSavedGifs, UpdateBotInlineQuery, UpdateBotInlineSend, UpdateEditChannelMessage, UpdateChannelPinnedMessage, UpdateBotCallbackQuery, UpdateEditMessage, UpdateInlineBotCallbackQuery, UpdateReadChannelOutbox, UpdateDraftMessage, UpdateReadFeaturedStickers, UpdateRecentStickers, UpdateConfig, UpdatePtsChanged, UpdateChannelWebPage, UpdateDialogPinned, UpdatePinnedDialogs, UpdateBotWebhookJSON, UpdateBotWebhookJSONQuery, UpdateBotShippingQuery, UpdateBotPrecheckoutQuery, UpdatePhoneCall.
        """
        super().__init__()

        self.chat_id = chat_id
        self.enabled = enabled
        self.version = version

    def on_send(self, writer):
        writer.write_int(UpdateChatAdmins.constructor_id, signed=False)
        writer.write_int(self.chat_id)
        writer.tgwrite_bool(self.enabled)
        writer.write_int(self.version)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateChatAdmins(None, None, None)

    def on_response(self, reader):
        self.chat_id = reader.read_int()
        self.enabled = reader.tgread_bool()
        self.version = reader.read_int()

    def __repr__(self):
        return 'updateChatAdmins#6e947941 chat_id:int enabled:Bool version:int = Update'

    def __str__(self):
        return '(updateChatAdmins (ID: 0x6e947941) = (chat_id={}, enabled={}, version={}))'.format(str(self.chat_id), str(self.enabled), str(self.version))



from .privacy_rules import PrivacyRules
from .authorizations import Authorizations
from .no_password import NoPassword
from .password import Password
from .password_settings import PasswordSettings
from .password_input_settings import PasswordInputSettings
from .tmp_password import TmpPassword



from ....tl.mtproto_request import MTProtoRequest


class Difference(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updates.difference#00f49ca0 new_messages:Vector<Message> new_encrypted_messages:Vector<EncryptedMessage> other_updates:Vector<Update> chats:Vector<Chat> users:Vector<User> state:updates.State = updates.Difference"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xf49ca0

    def __init__(self, new_messages, new_encrypted_messages, other_updates, chats, users, state):
        """
        :param new_messages: Telegram type: «Message». Must be a list.
        :param new_encrypted_messages: Telegram type: «EncryptedMessage». Must be a list.
        :param other_updates: Telegram type: «Update». Must be a list.
        :param chats: Telegram type: «Chat». Must be a list.
        :param users: Telegram type: «User». Must be a list.
        :param state: Telegram type: «updates.State».

        Constructor for updates.Difference: Instance of either DifferenceEmpty, Difference, DifferenceSlice, DifferenceTooLong.
        """
        super().__init__()

        self.new_messages = new_messages
        self.new_encrypted_messages = new_encrypted_messages
        self.other_updates = other_updates
        self.chats = chats
        self.users = users
        self.state = state

    def on_send(self, writer):
        writer.write_int(Difference.constructor_id, signed=False)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.new_messages))
        for new_messages_item in self.new_messages:
            new_messages_item.on_send(writer)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.new_encrypted_messages))
        for new_encrypted_messages_item in self.new_encrypted_messages:
            new_encrypted_messages_item.on_send(writer)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.other_updates))
        for other_updates_item in self.other_updates:
            other_updates_item.on_send(writer)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.chats))
        for chats_item in self.chats:
            chats_item.on_send(writer)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.users))
        for users_item in self.users:
            users_item.on_send(writer)

        self.state.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return Difference(None, None, None, None, None, None)

    def on_response(self, reader):
        reader.read_int()  # Vector's constructor ID
        self.new_messages = []  # Initialize an empty list
        new_messages_len = reader.read_int()
        for _ in range(new_messages_len):
            new_messages_item = reader.tgread_object()
            self.new_messages.append(new_messages_item)

        reader.read_int()  # Vector's constructor ID
        self.new_encrypted_messages = []  # Initialize an empty list
        new_encrypted_messages_len = reader.read_int()
        for _ in range(new_encrypted_messages_len):
            new_encrypted_messages_item = reader.tgread_object()
            self.new_encrypted_messages.append(new_encrypted_messages_item)

        reader.read_int()  # Vector's constructor ID
        self.other_updates = []  # Initialize an empty list
        other_updates_len = reader.read_int()
        for _ in range(other_updates_len):
            other_updates_item = reader.tgread_object()
            self.other_updates.append(other_updates_item)

        reader.read_int()  # Vector's constructor ID
        self.chats = []  # Initialize an empty list
        chats_len = reader.read_int()
        for _ in range(chats_len):
            chats_item = reader.tgread_object()
            self.chats.append(chats_item)

        reader.read_int()  # Vector's constructor ID
        self.users = []  # Initialize an empty list
        users_len = reader.read_int()
        for _ in range(users_len):
            users_item = reader.tgread_object()
            self.users.append(users_item)

        self.state = reader.tgread_object()

    def __repr__(self):
        return 'updates.difference#00f49ca0 new_messages:Vector<Message> new_encrypted_messages:Vector<EncryptedMessage> other_updates:Vector<Update> chats:Vector<Chat> users:Vector<User> state:updates.State = updates.Difference'

    def __str__(self):
        return '(updates.difference (ID: 0xf49ca0) = (new_messages={}, new_encrypted_messages={}, other_updates={}, chats={}, users={}, state={}))'.format(None if not self.new_messages else [str(_) for _ in self.new_messages], None if not self.new_encrypted_messages else [str(_) for _ in self.new_encrypted_messages], None if not self.other_updates else [str(_) for _ in self.other_updates], None if not self.chats else [str(_) for _ in self.chats], None if not self.users else [str(_) for _ in self.users], str(self.state))



from ....tl.mtproto_request import MTProtoRequest


class RequestEncryptionRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.requestEncryption#f64daf43 user_id:InputUser random_id:int g_a:bytes = EncryptedChat"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xf64daf43

    def __init__(self, user_id, random_id, g_a):
        """
        :param user_id: Telegram type: «InputUser».
        :param random_id: Telegram type: «int».
        :param g_a: Telegram type: «bytes».

        :returns EncryptedChat: Instance of either EncryptedChatEmpty, EncryptedChatWaiting, EncryptedChatRequested, EncryptedChat, EncryptedChatDiscarded.
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.user_id = user_id
        self.random_id = random_id
        self.g_a = g_a

    def on_send(self, writer):
        writer.write_int(RequestEncryptionRequest.constructor_id, signed=False)
        self.user_id.on_send(writer)
        writer.write_int(self.random_id)
        writer.tgwrite_bytes(self.g_a)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return RequestEncryptionRequest(None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.requestEncryption#f64daf43 user_id:InputUser random_id:int g_a:bytes = EncryptedChat'

    def __str__(self):
        return '(messages.requestEncryption (ID: 0xf64daf43) = (user_id={}, random_id={}, g_a={}))'.format(str(self.user_id), str(self.random_id), str(self.g_a))



from ...tl.mtproto_request import MTProtoRequest


class InputBotInlineResult(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    inputBotInlineResult#2cbbe15a flags:# id:string type:string title:flags.1?string description:flags.2?string url:flags.3?string thumb_url:flags.4?string content_url:flags.5?string content_type:flags.5?string w:flags.6?int h:flags.6?int duration:flags.7?int send_message:InputBotInlineMessage = InputBotInlineResult"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x2cbbe15a

    def __init__(self, id, type, send_message, title=None, description=None, url=None, thumb_url=None, content_url=None, content_type=None, w=None, h=None, duration=None):
        """
        :param id: Telegram type: «string».
        :param type: Telegram type: «string».
        :param title: Telegram type: «string».
        :param description: Telegram type: «string».
        :param url: Telegram type: «string».
        :param thumb_url: Telegram type: «string».
        :param content_url: Telegram type: «string».
        :param content_type: Telegram type: «string».
        :param w: Telegram type: «int».
        :param h: Telegram type: «int».
        :param duration: Telegram type: «int».
        :param send_message: Telegram type: «InputBotInlineMessage».

        Constructor for InputBotInlineResult: Instance of either InputBotInlineResult, InputBotInlineResultPhoto, InputBotInlineResultDocument, InputBotInlineResultGame.
        """
        super().__init__()

        self.id = id
        self.type = type
        self.title = title
        self.description = description
        self.url = url
        self.thumb_url = thumb_url
        self.content_url = content_url
        self.content_type = content_type
        self.w = w
        self.h = h
        self.duration = duration
        self.send_message = send_message

    def on_send(self, writer):
        writer.write_int(InputBotInlineResult.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 1) if self.title else 0
        flags |= (1 << 2) if self.description else 0
        flags |= (1 << 3) if self.url else 0
        flags |= (1 << 4) if self.thumb_url else 0
        flags |= (1 << 5) if self.content_url else 0
        flags |= (1 << 5) if self.content_type else 0
        flags |= (1 << 6) if self.w else 0
        flags |= (1 << 6) if self.h else 0
        flags |= (1 << 7) if self.duration else 0
        writer.write_int(flags)

        writer.tgwrite_string(self.id)
        writer.tgwrite_string(self.type)
        if self.title:
            writer.tgwrite_string(self.title)

        if self.description:
            writer.tgwrite_string(self.description)

        if self.url:
            writer.tgwrite_string(self.url)

        if self.thumb_url:
            writer.tgwrite_string(self.thumb_url)

        if self.content_url:
            writer.tgwrite_string(self.content_url)

        if self.content_type:
            writer.tgwrite_string(self.content_type)

        if self.w:
            writer.write_int(self.w)

        if self.h:
            writer.write_int(self.h)

        if self.duration:
            writer.write_int(self.duration)

        self.send_message.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InputBotInlineResult(None, None, None, None, None, None, None, None, None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        self.id = reader.tgread_string()
        self.type = reader.tgread_string()
        if (flags & (1 << 1)) != 0:
            self.title = reader.tgread_string()

        if (flags & (1 << 2)) != 0:
            self.description = reader.tgread_string()

        if (flags & (1 << 3)) != 0:
            self.url = reader.tgread_string()

        if (flags & (1 << 4)) != 0:
            self.thumb_url = reader.tgread_string()

        if (flags & (1 << 5)) != 0:
            self.content_url = reader.tgread_string()

        if (flags & (1 << 5)) != 0:
            self.content_type = reader.tgread_string()

        if (flags & (1 << 6)) != 0:
            self.w = reader.read_int()

        if (flags & (1 << 6)) != 0:
            self.h = reader.read_int()

        if (flags & (1 << 7)) != 0:
            self.duration = reader.read_int()

        self.send_message = reader.tgread_object()

    def __repr__(self):
        return 'inputBotInlineResult#2cbbe15a flags:# id:string type:string title:flags.1?string description:flags.2?string url:flags.3?string thumb_url:flags.4?string content_url:flags.5?string content_type:flags.5?string w:flags.6?int h:flags.6?int duration:flags.7?int send_message:InputBotInlineMessage = InputBotInlineResult'

    def __str__(self):
        return '(inputBotInlineResult (ID: 0x2cbbe15a) = (id={}, type={}, title={}, description={}, url={}, thumb_url={}, content_url={}, content_type={}, w={}, h={}, duration={}, send_message={}))'.format(str(self.id), str(self.type), str(self.title), str(self.description), str(self.url), str(self.thumb_url), str(self.content_url), str(self.content_type), str(self.w), str(self.h), str(self.duration), str(self.send_message))



from ...tl.mtproto_request import MTProtoRequest


class UpdateShortSentMessage(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateShortSentMessage#11f1331c flags:# out:flags.1?true id:int pts:int pts_count:int date:date media:flags.9?MessageMedia entities:flags.7?Vector<MessageEntity> = Updates"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x11f1331c

    def __init__(self, id, pts, pts_count, date, out=None, media=None, entities=None):
        """
        :param out: Telegram type: «true».
        :param id: Telegram type: «int».
        :param pts: Telegram type: «int».
        :param pts_count: Telegram type: «int».
        :param date: Telegram type: «date».
        :param media: Telegram type: «MessageMedia».
        :param entities: Telegram type: «MessageEntity». Must be a list.

        Constructor for Updates: Instance of either UpdatesTooLong, UpdateShortMessage, UpdateShortChatMessage, UpdateShort, UpdatesCombined, UpdatesTg, UpdateShortSentMessage.
        """
        super().__init__()

        self.out = out
        self.id = id
        self.pts = pts
        self.pts_count = pts_count
        self.date = date
        self.media = media
        self.entities = entities

    def on_send(self, writer):
        writer.write_int(UpdateShortSentMessage.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 1) if self.out else 0
        flags |= (1 << 9) if self.media else 0
        flags |= (1 << 7) if self.entities else 0
        writer.write_int(flags)

        writer.write_int(self.id)
        writer.write_int(self.pts)
        writer.write_int(self.pts_count)
        writer.tgwrite_date(self.date)
        if self.media:
            self.media.on_send(writer)

        if self.entities:
            writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
            writer.write_int(len(self.entities))
            for entities_item in self.entities:
                if entities_item:
                    entities_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateShortSentMessage(None, None, None, None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 1)) != 0:
            self.out = True  # Arbitrary not-None value, no need to read since it is a flag

        self.id = reader.read_int()
        self.pts = reader.read_int()
        self.pts_count = reader.read_int()
        self.date = reader.tgread_date()
        if (flags & (1 << 9)) != 0:
            self.media = reader.tgread_object()

        if (flags & (1 << 7)) != 0:
            reader.read_int()  # Vector's constructor ID
            self.entities = []  # Initialize an empty list
            entities_len = reader.read_int()
            for _ in range(entities_len):
                entities_item = reader.tgread_object()
                self.entities.append(entities_item)

    def __repr__(self):
        return 'updateShortSentMessage#11f1331c flags:# out:flags.1?true id:int pts:int pts_count:int date:date media:flags.9?MessageMedia entities:flags.7?Vector<MessageEntity> = Updates'

    def __str__(self):
        return '(updateShortSentMessage (ID: 0x11f1331c) = (out={}, id={}, pts={}, pts_count={}, date={}, media={}, entities={}))'.format(str(self.out), str(self.id), str(self.pts), str(self.pts_count), str(self.date), str(self.media), None if not self.entities else [str(_) for _ in self.entities])



from ...tl.mtproto_request import MTProtoRequest


class PhoneConnection(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    phoneConnection#9d4c17c0 id:long ip:string ipv6:string port:int peer_tag:bytes = PhoneConnection"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x9d4c17c0

    def __init__(self, id, ip, ipv6, port, peer_tag):
        """
        :param id: Telegram type: «long».
        :param ip: Telegram type: «string».
        :param ipv6: Telegram type: «string».
        :param port: Telegram type: «int».
        :param peer_tag: Telegram type: «bytes».

        Constructor for PhoneConnection: Instance of PhoneConnection.
        """
        super().__init__()

        self.id = id
        self.ip = ip
        self.ipv6 = ipv6
        self.port = port
        self.peer_tag = peer_tag

    def on_send(self, writer):
        writer.write_int(PhoneConnection.constructor_id, signed=False)
        writer.write_long(self.id)
        writer.tgwrite_string(self.ip)
        writer.tgwrite_string(self.ipv6)
        writer.write_int(self.port)
        writer.tgwrite_bytes(self.peer_tag)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return PhoneConnection(None, None, None, None, None)

    def on_response(self, reader):
        self.id = reader.read_long()
        self.ip = reader.tgread_string()
        self.ipv6 = reader.tgread_string()
        self.port = reader.read_int()
        self.peer_tag = reader.tgread_bytes()

    def __repr__(self):
        return 'phoneConnection#9d4c17c0 id:long ip:string ipv6:string port:int peer_tag:bytes = PhoneConnection'

    def __str__(self):
        return '(phoneConnection (ID: 0x9d4c17c0) = (id={}, ip={}, ipv6={}, port={}, peer_tag={}))'.format(str(self.id), str(self.ip), str(self.ipv6), str(self.port), str(self.peer_tag))



from ...tl.mtproto_request import MTProtoRequest


class MessageActionChatAddUser(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messageActionChatAddUser#488a7337 users:Vector<int> = MessageAction"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x488a7337

    def __init__(self, users):
        """
        :param users: Telegram type: «int». Must be a list.

        Constructor for MessageAction: Instance of either MessageActionEmpty, MessageActionChatCreate, MessageActionChatEditTitle, MessageActionChatEditPhoto, MessageActionChatDeletePhoto, MessageActionChatAddUser, MessageActionChatDeleteUser, MessageActionChatJoinedByLink, MessageActionChannelCreate, MessageActionChatMigrateTo, MessageActionChannelMigrateFrom, MessageActionPinMessage, MessageActionHistoryClear, MessageActionGameScore, MessageActionPaymentSentMe, MessageActionPaymentSent, MessageActionPhoneCall.
        """
        super().__init__()

        self.users = users

    def on_send(self, writer):
        writer.write_int(MessageActionChatAddUser.constructor_id, signed=False)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.users))
        for users_item in self.users:
            writer.write_int(users_item)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return MessageActionChatAddUser(None)

    def on_response(self, reader):
        reader.read_int()  # Vector's constructor ID
        self.users = []  # Initialize an empty list
        users_len = reader.read_int()
        for _ in range(users_len):
            users_item = reader.read_int()
            self.users.append(users_item)

    def __repr__(self):
        return 'messageActionChatAddUser#488a7337 users:Vector<int> = MessageAction'

    def __str__(self):
        return '(messageActionChatAddUser (ID: 0x488a7337) = (users={}))'.format(None if not self.users else [str(_) for _ in self.users])



from ....tl.mtproto_request import MTProtoRequest


class ReportPeerRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    account.reportPeer#ae189d5f peer:InputPeer reason:ReportReason = Bool"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xae189d5f

    def __init__(self, peer, reason):
        """
        :param peer: Telegram type: «InputPeer».
        :param reason: Telegram type: «ReportReason».

        :returns Bool: This type has no constructors.
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.peer = peer
        self.reason = reason

    def on_send(self, writer):
        writer.write_int(ReportPeerRequest.constructor_id, signed=False)
        self.peer.on_send(writer)
        self.reason.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ReportPeerRequest(None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'account.reportPeer#ae189d5f peer:InputPeer reason:ReportReason = Bool'

    def __str__(self):
        return '(account.reportPeer (ID: 0xae189d5f) = (peer={}, reason={}))'.format(str(self.peer), str(self.reason))



from ...tl.mtproto_request import MTProtoRequest


class UpdateEncryption(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateEncryption#b4a2e88d chat:EncryptedChat date:date = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xb4a2e88d

    def __init__(self, chat, date):
        """
        :param chat: Telegram type: «EncryptedChat».
        :param date: Telegram type: «date».

        Constructor for Update: Instance of either UpdateNewMessage, UpdateMessageID, UpdateDeleteMessages, UpdateUserTyping, UpdateChatUserTyping, UpdateChatParticipants, UpdateUserStatus, UpdateUserName, UpdateUserPhoto, UpdateContactRegistered, UpdateContactLink, UpdateNewEncryptedMessage, UpdateEncryptedChatTyping, UpdateEncryption, UpdateEncryptedMessagesRead, UpdateChatParticipantAdd, UpdateChatParticipantDelete, UpdateDcOptions, UpdateUserBlocked, UpdateNotifySettings, UpdateServiceNotification, UpdatePrivacy, UpdateUserPhone, UpdateReadHistoryInbox, UpdateReadHistoryOutbox, UpdateWebPage, UpdateReadMessagesContents, UpdateChannelTooLong, UpdateChannel, UpdateNewChannelMessage, UpdateReadChannelInbox, UpdateDeleteChannelMessages, UpdateChannelMessageViews, UpdateChatAdmins, UpdateChatParticipantAdmin, UpdateNewStickerSet, UpdateStickerSetsOrder, UpdateStickerSets, UpdateSavedGifs, UpdateBotInlineQuery, UpdateBotInlineSend, UpdateEditChannelMessage, UpdateChannelPinnedMessage, UpdateBotCallbackQuery, UpdateEditMessage, UpdateInlineBotCallbackQuery, UpdateReadChannelOutbox, UpdateDraftMessage, UpdateReadFeaturedStickers, UpdateRecentStickers, UpdateConfig, UpdatePtsChanged, UpdateChannelWebPage, UpdateDialogPinned, UpdatePinnedDialogs, UpdateBotWebhookJSON, UpdateBotWebhookJSONQuery, UpdateBotShippingQuery, UpdateBotPrecheckoutQuery, UpdatePhoneCall.
        """
        super().__init__()

        self.chat = chat
        self.date = date

    def on_send(self, writer):
        writer.write_int(UpdateEncryption.constructor_id, signed=False)
        self.chat.on_send(writer)
        writer.tgwrite_date(self.date)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateEncryption(None, None)

    def on_response(self, reader):
        self.chat = reader.tgread_object()
        self.date = reader.tgread_date()

    def __repr__(self):
        return 'updateEncryption#b4a2e88d chat:EncryptedChat date:date = Update'

    def __str__(self):
        return '(updateEncryption (ID: 0xb4a2e88d) = (chat={}, date={}))'.format(str(self.chat), str(self.date))



from ...tl.mtproto_request import MTProtoRequest


class InputPhoneContact(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    inputPhoneContact#f392b7f4 client_id:long phone:string first_name:string last_name:string = InputContact"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xf392b7f4

    def __init__(self, client_id, phone, first_name, last_name):
        """
        :param client_id: Telegram type: «long».
        :param phone: Telegram type: «string».
        :param first_name: Telegram type: «string».
        :param last_name: Telegram type: «string».

        Constructor for InputContact: Instance of InputPhoneContact.
        """
        super().__init__()

        self.client_id = client_id
        self.phone = phone
        self.first_name = first_name
        self.last_name = last_name

    def on_send(self, writer):
        writer.write_int(InputPhoneContact.constructor_id, signed=False)
        writer.write_long(self.client_id)
        writer.tgwrite_string(self.phone)
        writer.tgwrite_string(self.first_name)
        writer.tgwrite_string(self.last_name)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InputPhoneContact(None, None, None, None)

    def on_response(self, reader):
        self.client_id = reader.read_long()
        self.phone = reader.tgread_string()
        self.first_name = reader.tgread_string()
        self.last_name = reader.tgread_string()

    def __repr__(self):
        return 'inputPhoneContact#f392b7f4 client_id:long phone:string first_name:string last_name:string = InputContact'

    def __str__(self):
        return '(inputPhoneContact (ID: 0xf392b7f4) = (client_id={}, phone={}, first_name={}, last_name={}))'.format(str(self.client_id), str(self.phone), str(self.first_name), str(self.last_name))



from ...tl.mtproto_request import MTProtoRequest


class PaymentCharge(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    paymentCharge#ea02c27e id:string provider_charge_id:string = PaymentCharge"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xea02c27e

    def __init__(self, id, provider_charge_id):
        """
        :param id: Telegram type: «string».
        :param provider_charge_id: Telegram type: «string».

        Constructor for PaymentCharge: Instance of PaymentCharge.
        """
        super().__init__()

        self.id = id
        self.provider_charge_id = provider_charge_id

    def on_send(self, writer):
        writer.write_int(PaymentCharge.constructor_id, signed=False)
        writer.tgwrite_string(self.id)
        writer.tgwrite_string(self.provider_charge_id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return PaymentCharge(None, None)

    def on_response(self, reader):
        self.id = reader.tgread_string()
        self.provider_charge_id = reader.tgread_string()

    def __repr__(self):
        return 'paymentCharge#ea02c27e id:string provider_charge_id:string = PaymentCharge'

    def __str__(self):
        return '(paymentCharge (ID: 0xea02c27e) = (id={}, provider_charge_id={}))'.format(str(self.id), str(self.provider_charge_id))



from ....tl.mtproto_request import MTProtoRequest


class PaymentForm(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    payments.paymentForm#3f56aea3 flags:# can_save_credentials:flags.2?true password_missing:flags.3?true bot_id:int invoice:Invoice provider_id:int url:string native_provider:flags.4?string native_params:flags.4?DataJSON saved_info:flags.0?PaymentRequestedInfo saved_credentials:flags.1?PaymentSavedCredentials users:Vector<User> = payments.PaymentForm"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x3f56aea3

    def __init__(self, bot_id, invoice, provider_id, url, users, can_save_credentials=None, password_missing=None, native_provider=None, native_params=None, saved_info=None, saved_credentials=None):
        """
        :param can_save_credentials: Telegram type: «true».
        :param password_missing: Telegram type: «true».
        :param bot_id: Telegram type: «int».
        :param invoice: Telegram type: «Invoice».
        :param provider_id: Telegram type: «int».
        :param url: Telegram type: «string».
        :param native_provider: Telegram type: «string».
        :param native_params: Telegram type: «DataJSON».
        :param saved_info: Telegram type: «PaymentRequestedInfo».
        :param saved_credentials: Telegram type: «PaymentSavedCredentials».
        :param users: Telegram type: «User». Must be a list.

        Constructor for payments.PaymentForm: Instance of PaymentForm.
        """
        super().__init__()

        self.can_save_credentials = can_save_credentials
        self.password_missing = password_missing
        self.bot_id = bot_id
        self.invoice = invoice
        self.provider_id = provider_id
        self.url = url
        self.native_provider = native_provider
        self.native_params = native_params
        self.saved_info = saved_info
        self.saved_credentials = saved_credentials
        self.users = users

    def on_send(self, writer):
        writer.write_int(PaymentForm.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 2) if self.can_save_credentials else 0
        flags |= (1 << 3) if self.password_missing else 0
        flags |= (1 << 4) if self.native_provider else 0
        flags |= (1 << 4) if self.native_params else 0
        flags |= (1 << 0) if self.saved_info else 0
        flags |= (1 << 1) if self.saved_credentials else 0
        writer.write_int(flags)

        writer.write_int(self.bot_id)
        self.invoice.on_send(writer)
        writer.write_int(self.provider_id)
        writer.tgwrite_string(self.url)
        if self.native_provider:
            writer.tgwrite_string(self.native_provider)

        if self.native_params:
            self.native_params.on_send(writer)

        if self.saved_info:
            self.saved_info.on_send(writer)

        if self.saved_credentials:
            self.saved_credentials.on_send(writer)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.users))
        for users_item in self.users:
            users_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return PaymentForm(None, None, None, None, None, None, None, None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 2)) != 0:
            self.can_save_credentials = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 3)) != 0:
            self.password_missing = True  # Arbitrary not-None value, no need to read since it is a flag

        self.bot_id = reader.read_int()
        self.invoice = reader.tgread_object()
        self.provider_id = reader.read_int()
        self.url = reader.tgread_string()
        if (flags & (1 << 4)) != 0:
            self.native_provider = reader.tgread_string()

        if (flags & (1 << 4)) != 0:
            self.native_params = reader.tgread_object()

        if (flags & (1 << 0)) != 0:
            self.saved_info = reader.tgread_object()

        if (flags & (1 << 1)) != 0:
            self.saved_credentials = reader.tgread_object()

        reader.read_int()  # Vector's constructor ID
        self.users = []  # Initialize an empty list
        users_len = reader.read_int()
        for _ in range(users_len):
            users_item = reader.tgread_object()
            self.users.append(users_item)

    def __repr__(self):
        return 'payments.paymentForm#3f56aea3 flags:# can_save_credentials:flags.2?true password_missing:flags.3?true bot_id:int invoice:Invoice provider_id:int url:string native_provider:flags.4?string native_params:flags.4?DataJSON saved_info:flags.0?PaymentRequestedInfo saved_credentials:flags.1?PaymentSavedCredentials users:Vector<User> = payments.PaymentForm'

    def __str__(self):
        return '(payments.paymentForm (ID: 0x3f56aea3) = (can_save_credentials={}, password_missing={}, bot_id={}, invoice={}, provider_id={}, url={}, native_provider={}, native_params={}, saved_info={}, saved_credentials={}, users={}))'.format(str(self.can_save_credentials), str(self.password_missing), str(self.bot_id), str(self.invoice), str(self.provider_id), str(self.url), str(self.native_provider), str(self.native_params), str(self.saved_info), str(self.saved_credentials), None if not self.users else [str(_) for _ in self.users])



from ...tl.mtproto_request import MTProtoRequest


class NewSessionCreated(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    new_session_created#9ec20908 first_msg_id:long unique_id:long server_salt:long = NewSession"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x9ec20908

    def __init__(self, first_msg_id, unique_id, server_salt):
        """
        :param first_msg_id: Telegram type: «long».
        :param unique_id: Telegram type: «long».
        :param server_salt: Telegram type: «long».

        Constructor for NewSession: Instance of NewSessionCreated.
        """
        super().__init__()

        self.first_msg_id = first_msg_id
        self.unique_id = unique_id
        self.server_salt = server_salt

    def on_send(self, writer):
        writer.write_int(NewSessionCreated.constructor_id, signed=False)
        writer.write_long(self.first_msg_id)
        writer.write_long(self.unique_id)
        writer.write_long(self.server_salt)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return NewSessionCreated(None, None, None)

    def on_response(self, reader):
        self.first_msg_id = reader.read_long()
        self.unique_id = reader.read_long()
        self.server_salt = reader.read_long()

    def __repr__(self):
        return 'new_session_created#9ec20908 first_msg_id:long unique_id:long server_salt:long = NewSession'

    def __str__(self):
        return '(new_session_created (ID: 0x9ec20908) = (first_msg_id={}, unique_id={}, server_salt={}))'.format(str(self.first_msg_id), str(self.unique_id), str(self.server_salt))



from .mtproto_plain_sender import MtProtoPlainSender
from .tcp_client import TcpClient
from .authenticator import do_authentication
from .mtproto_sender import MtProtoSender
from .tcp_transport import TcpTransport



from ...tl.mtproto_request import MTProtoRequest


class MessageMediaContact(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messageMediaContact#5e7d2f39 phone_number:string first_name:string last_name:string user_id:int = MessageMedia"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x5e7d2f39

    def __init__(self, phone_number, first_name, last_name, user_id):
        """
        :param phone_number: Telegram type: «string».
        :param first_name: Telegram type: «string».
        :param last_name: Telegram type: «string».
        :param user_id: Telegram type: «int».

        Constructor for MessageMedia: Instance of either MessageMediaEmpty, MessageMediaPhoto, MessageMediaGeo, MessageMediaContact, MessageMediaUnsupported, MessageMediaDocument, MessageMediaWebPage, MessageMediaVenue, MessageMediaGame, MessageMediaInvoice.
        """
        super().__init__()

        self.phone_number = phone_number
        self.first_name = first_name
        self.last_name = last_name
        self.user_id = user_id

    def on_send(self, writer):
        writer.write_int(MessageMediaContact.constructor_id, signed=False)
        writer.tgwrite_string(self.phone_number)
        writer.tgwrite_string(self.first_name)
        writer.tgwrite_string(self.last_name)
        writer.write_int(self.user_id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return MessageMediaContact(None, None, None, None)

    def on_response(self, reader):
        self.phone_number = reader.tgread_string()
        self.first_name = reader.tgread_string()
        self.last_name = reader.tgread_string()
        self.user_id = reader.read_int()

    def __repr__(self):
        return 'messageMediaContact#5e7d2f39 phone_number:string first_name:string last_name:string user_id:int = MessageMedia'

    def __str__(self):
        return '(messageMediaContact (ID: 0x5e7d2f39) = (phone_number={}, first_name={}, last_name={}, user_id={}))'.format(str(self.phone_number), str(self.first_name), str(self.last_name), str(self.user_id))



from ....tl.mtproto_request import MTProtoRequest


class CodeTypeFlashCall(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    auth.codeTypeFlashCall#226ccefb  = auth.CodeType"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x226ccefb

    def __init__(self):
        super().__init__()

    def on_send(self, writer):
        writer.write_int(CodeTypeFlashCall.constructor_id, signed=False)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return CodeTypeFlashCall()

    def on_response(self, reader):
        pass

    def __repr__(self):
        return 'auth.codeTypeFlashCall#226ccefb  = auth.CodeType'

    def __str__(self):
        return '(auth.codeTypeFlashCall (ID: 0x226ccefb) = ())'.format()



from ....tl.mtproto_request import MTProtoRequest


class SearchGifsRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.searchGifs#bf9a776b q:string offset:int = messages.FoundGifs"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xbf9a776b

    def __init__(self, q, offset):
        """
        :param q: Telegram type: «string».
        :param offset: Telegram type: «int».

        :returns messages.FoundGifs: Instance of FoundGifs.
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.q = q
        self.offset = offset

    def on_send(self, writer):
        writer.write_int(SearchGifsRequest.constructor_id, signed=False)
        writer.tgwrite_string(self.q)
        writer.write_int(self.offset)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return SearchGifsRequest(None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.searchGifs#bf9a776b q:string offset:int = messages.FoundGifs'

    def __str__(self):
        return '(messages.searchGifs (ID: 0xbf9a776b) = (q={}, offset={}))'.format(str(self.q), str(self.offset))



import shutil
from getpass import getpass

from . import TelegramClient
from .errors import RPCError
from .tl.types import UpdateShortChatMessage, UpdateShortMessage
from .utils import get_display_name

# Get the (current) number of lines in the terminal
cols, rows = shutil.get_terminal_size()


def sprint(string, *args, **kwargs):
    """Safe Print (handle UnicodeEncodeErrors on some terminals)"""
    try:
        print(string, *args, **kwargs)
    except UnicodeEncodeError:
        string = string.encode('utf-8', errors='ignore')\
                       .decode('ascii', errors='ignore')
        print(string, *args, **kwargs)


def print_title(title):
    # Clear previous window
    print('\n')
    print('=={}=='.format('=' * len(title)))
    sprint('= {} ='.format(title))
    print('=={}=='.format('=' * len(title)))


def bytes_to_string(byte_count):
    """Converts a byte count to a string (in KB, MB...)"""
    suffix_index = 0
    while byte_count >= 1024:
        byte_count /= 1024
        suffix_index += 1

    return '{:.2f}{}'.format(byte_count,
                             [' bytes', 'KB', 'MB', 'GB', 'TB'][suffix_index])


class InteractiveTelegramClient(TelegramClient):
    def __init__(self, session_user_id, user_phone, api_id, api_hash, proxy=None):
        print_title('Initialization')

        print('Initializing interactive example...')
        super().__init__(session_user_id, api_id, api_hash, proxy)

        # Store all the found media in memory here,
        # so it can be downloaded if the user wants
        self.found_media = set()

        print('Connecting to Telegram servers...')
        if not self.connect():
            print('Initial connection failed. Retrying...')
            if not self.connect():
                print('Could not connect to Telegram servers.')
                return

        # Then, ensure we're authorized and have access
        if not self.is_user_authorized():
            print('First run. Sending code request...')
            self.send_code_request(user_phone)

            code_ok = False
            while not code_ok:
                code = input('Enter the code you just received: ')
                try:
                    code_ok = self.sign_in(user_phone, code)

                # Two-step verification may be enabled
                except RPCError as e:
                    if e.password_required:
                        pw = getpass(
                            'Two step verification is enabled. Please enter your password: ')
                        code_ok = self.sign_in(password=pw)
                    else:
                        raise

    def run(self):
        # Listen for updates
        self.add_update_handler(self.update_handler)

        # Enter a while loop to chat as long as the user wants
        while True:
            # Retrieve the top dialogs
            dialog_count = 10

            # Entities represent the user, chat or channel
            # corresponding to the dialog on the same index
            dialogs, entities = self.get_dialogs(dialog_count)

            i = None
            while i is None:
                print_title('Dialogs window')

                # Display them so the user can choose
                for i, entity in enumerate(entities):
                    i += 1  # 1-based index
                    sprint('{}. {}'.format(i, get_display_name(entity)))

                # Let the user decide who they want to talk to
                print()
                print('> Who do you want to send messages to?')
                print('> Available commands:')
                print('  !q: Quits the dialogs window and exits.')
                print('  !l: Logs out, terminating this session.')
                print()
                i = input('Enter dialog ID or a command: ')
                if i == '!q':
                    return
                if i == '!l':
                    self.log_out()
                    return

                try:
                    i = int(i if i else 0) - 1
                    # Ensure it is inside the bounds, otherwise set to None and retry
                    if not 0 <= i < dialog_count:
                        i = None
                except ValueError:
                    i = None

            # Retrieve the selected user (or chat, or channel)
            entity = entities[i]

            # Show some information
            print_title('Chat with "{}"'.format(get_display_name(entity)))
            print('Available commands:')
            print('  !q: Quits the current chat.')
            print('  !Q: Quits the current chat and exits.')
            print(
                '  !h: prints the latest messages (message History) of the chat.')
            print(
                '  !up <path>: Uploads and sends a Photo located at the given path.')
            print(
                '  !uf <path>: Uploads and sends a File document located at the given path.')
            print(
                '  !dm <msg-id>: Downloads the given message Media (if any).')
            print('  !dp: Downloads the current dialog Profile picture.')
            print()

            # And start a while loop to chat
            while True:
                msg = input('Enter a message: ')
                # Quit
                if msg == '!q':
                    break
                elif msg == '!Q':
                    return

                # History
                elif msg == '!h':
                    # First retrieve the messages and some information
                    total_count, messages, senders = self.get_message_history(
                        entity, limit=10)
                    # Iterate over all (in reverse order so the latest appears the last in the console)
                    # and print them in "[hh:mm] Sender: Message" text format
                    for msg, sender in zip(
                            reversed(messages), reversed(senders)):
                        # Get the name of the sender if any
                        name = sender.first_name if sender else '???'

                        # Format the message content
                        if getattr(msg, 'media', None):
                            self.found_media.add(msg)
                            content = '<{}> {}'.format(  # The media may or may not have a caption
                                msg.media.__class__.__name__,
                                getattr(msg.media, 'caption', ''))
                        elif hasattr(msg, 'message'):
                            content = msg.message
                        elif hasattr(msg, 'action'):
                            content = str(msg.action)
                        else:
                            # Unknown message, simply print its class name
                            content = msg.__class__.__name__

                        # And print it to the user
                        sprint('[{}:{}] (ID={}) {}: {}'.format(
                               msg.date.hour, msg.date.minute, msg.id, name,
                               content))

                # Send photo
                elif msg.startswith('!up '):
                    # Slice the message to get the path
                    self.send_photo(path=msg[len('!up '):], entity=entity)

                # Send file (document)
                elif msg.startswith('!uf '):
                    # Slice the message to get the path
                    self.send_document(path=msg[len('!uf '):], entity=entity)

                # Download media
                elif msg.startswith('!dm '):
                    # Slice the message to get message ID
                    self.download_media(msg[len('!dm '):])

                # Download profile photo
                elif msg == '!dp':
                    output = str('usermedia/propic_{}'.format(entity.id))
                    print('Downloading profile picture...')
                    success = self.download_profile_photo(entity.photo, output)
                    if success:
                        print('Profile picture downloaded to {}'.format(
                            output))
                    else:
                        print('No profile picture found for this user.')

                # Send chat message (if any)
                elif msg:
                    self.send_message(
                        entity, msg, markdown=True, no_web_page=True)

    def send_photo(self, path, entity):
        print('Uploading {}...'.format(path))
        input_file = self.upload_file(
            path, progress_callback=self.upload_progress_callback)

        # After we have the handle to the uploaded file, send it to our peer
        self.send_photo_file(input_file, entity)
        print('Photo sent!')

    def send_document(self, path, entity):
        print('Uploading {}...'.format(path))
        input_file = self.upload_file(
            path, progress_callback=self.upload_progress_callback)

        # After we have the handle to the uploaded file, send it to our peer
        self.send_document_file(input_file, entity)
        print('Document sent!')

    def download_media(self, media_id):
        try:
            # The user may have entered a non-integer string!
            msg_media_id = int(media_id)

            # Search the message ID
            for msg in self.found_media:
                if msg.id == msg_media_id:
                    # Let the output be the message ID
                    output = str('usermedia/{}'.format(msg_media_id))
                    print('Downloading media with name {}...'.format(output))
                    output = self.download_msg_media(
                        msg.media,
                        file_path=output,
                        progress_callback=self.download_progress_callback)
                    print('Media downloaded to {}!'.format(output))

        except ValueError:
            print('Invalid media ID given!')

    @staticmethod
    def download_progress_callback(downloaded_bytes, total_bytes):
        InteractiveTelegramClient.print_progress('Downloaded',
                                                 downloaded_bytes, total_bytes)

    @staticmethod
    def upload_progress_callback(uploaded_bytes, total_bytes):
        InteractiveTelegramClient.print_progress('Uploaded', uploaded_bytes,
                                                 total_bytes)

    @staticmethod
    def print_progress(progress_type, downloaded_bytes, total_bytes):
        print('{} {} out of {} ({:.2%})'.format(progress_type, bytes_to_string(
            downloaded_bytes), bytes_to_string(total_bytes), downloaded_bytes /
                                                total_bytes))

    @staticmethod
    def update_handler(update_object):
        if type(update_object) is UpdateShortMessage:
            if update_object.out:
                sprint('You sent {} to user #{}'.format(
                    update_object.message, update_object.user_id))
            else:
                sprint('[User #{} sent {}]'.format(
                    update_object.user_id, update_object.message))

        elif type(update_object) is UpdateShortChatMessage:
            if update_object.out:
                sprint('You sent {} to chat #{}'.format(
                    update_object.message, update_object.chat_id))
            else:
                sprint('[Chat #{}, user #{} sent {}]'.format(
                       update_object.chat_id, update_object.from_id,
                       update_object.message))



from ....tl.mtproto_request import MTProtoRequest


class GetParticipantsRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    channels.getParticipants#24d98f92 channel:InputChannel filter:ChannelParticipantsFilter offset:int limit:int = channels.ChannelParticipants"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x24d98f92

    def __init__(self, channel, filter, offset, limit):
        """
        :param channel: Telegram type: «InputChannel».
        :param filter: Telegram type: «ChannelParticipantsFilter».
        :param offset: Telegram type: «int».
        :param limit: Telegram type: «int».

        :returns channels.ChannelParticipants: Instance of ChannelParticipants.
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.channel = channel
        self.filter = filter
        self.offset = offset
        self.limit = limit

    def on_send(self, writer):
        writer.write_int(GetParticipantsRequest.constructor_id, signed=False)
        self.channel.on_send(writer)
        self.filter.on_send(writer)
        writer.write_int(self.offset)
        writer.write_int(self.limit)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return GetParticipantsRequest(None, None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'channels.getParticipants#24d98f92 channel:InputChannel filter:ChannelParticipantsFilter offset:int limit:int = channels.ChannelParticipants'

    def __str__(self):
        return '(channels.getParticipants (ID: 0x24d98f92) = (channel={}, filter={}, offset={}, limit={}))'.format(str(self.channel), str(self.filter), str(self.offset), str(self.limit))



from .req_pq import ReqPqRequest
from .req_dh_params import ReqDHParamsRequest
from .set_client_dh_params import SetClientDHParamsRequest
from .destroy_auth_key import DestroyAuthKeyRequest
from .rpc_drop_answer import RpcDropAnswerRequest
from .get_future_salts import GetFutureSaltsRequest
from .ping import PingRequest
from .ping_delay_disconnect import PingDelayDisconnectRequest
from .destroy_session import DestroySessionRequest
from .invoke_after_msg import InvokeAfterMsgRequest
from .invoke_after_msgs import InvokeAfterMsgsRequest
from .init_connection import InitConnectionRequest
from .invoke_with_layer import InvokeWithLayerRequest
from .invoke_without_updates import InvokeWithoutUpdatesRequest
from . import account, contacts, payments, auth, contest, messages, help, channels, bots, phone, photos, updates, upload, users



from ...tl.mtproto_request import MTProtoRequest


class ServerDHInnerData(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    server_DH_inner_data#b5890dba nonce:int128 server_nonce:int128 g:int dh_prime:string g_a:string server_time:int = Server_DH_inner_data"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xb5890dba

    def __init__(self, nonce, server_nonce, g, dh_prime, g_a, server_time):
        """
        :param nonce: Telegram type: «int128».
        :param server_nonce: Telegram type: «int128».
        :param g: Telegram type: «int».
        :param dh_prime: Telegram type: «string».
        :param g_a: Telegram type: «string».
        :param server_time: Telegram type: «int».

        Constructor for Server_DH_inner_data: Instance of ServerDHInnerData.
        """
        super().__init__()

        self.nonce = nonce
        self.server_nonce = server_nonce
        self.g = g
        self.dh_prime = dh_prime
        self.g_a = g_a
        self.server_time = server_time

    def on_send(self, writer):
        writer.write_int(ServerDHInnerData.constructor_id, signed=False)
        writer.write_large_int(self.nonce, bits=128)
        writer.write_large_int(self.server_nonce, bits=128)
        writer.write_int(self.g)
        writer.tgwrite_string(self.dh_prime)
        writer.tgwrite_string(self.g_a)
        writer.write_int(self.server_time)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ServerDHInnerData(None, None, None, None, None, None)

    def on_response(self, reader):
        self.nonce = reader.read_large_int(bits=128)
        self.server_nonce = reader.read_large_int(bits=128)
        self.g = reader.read_int()
        self.dh_prime = reader.tgread_string()
        self.g_a = reader.tgread_string()
        self.server_time = reader.read_int()

    def __repr__(self):
        return 'server_DH_inner_data#b5890dba nonce:int128 server_nonce:int128 g:int dh_prime:string g_a:string server_time:int = Server_DH_inner_data'

    def __str__(self):
        return '(server_DH_inner_data (ID: 0xb5890dba) = (nonce={}, server_nonce={}, g={}, dh_prime={}, g_a={}, server_time={}))'.format(str(self.nonce), str(self.server_nonce), str(self.g), str(self.dh_prime), str(self.g_a), str(self.server_time))



from ...tl.mtproto_request import MTProtoRequest


class MessageEntityUnknown(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messageEntityUnknown#bb92ba95 offset:int length:int = MessageEntity"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xbb92ba95

    def __init__(self, offset, length):
        """
        :param offset: Telegram type: «int».
        :param length: Telegram type: «int».

        Constructor for MessageEntity: Instance of either MessageEntityUnknown, MessageEntityMention, MessageEntityHashtag, MessageEntityBotCommand, MessageEntityUrl, MessageEntityEmail, MessageEntityBold, MessageEntityItalic, MessageEntityCode, MessageEntityPre, MessageEntityTextUrl, MessageEntityMentionName, InputMessageEntityMentionName.
        """
        super().__init__()

        self.offset = offset
        self.length = length

    def on_send(self, writer):
        writer.write_int(MessageEntityUnknown.constructor_id, signed=False)
        writer.write_int(self.offset)
        writer.write_int(self.length)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return MessageEntityUnknown(None, None)

    def on_response(self, reader):
        self.offset = reader.read_int()
        self.length = reader.read_int()

    def __repr__(self):
        return 'messageEntityUnknown#bb92ba95 offset:int length:int = MessageEntity'

    def __str__(self):
        return '(messageEntityUnknown (ID: 0xbb92ba95) = (offset={}, length={}))'.format(str(self.offset), str(self.length))



from ...tl.mtproto_request import MTProtoRequest


class InputDocumentFileLocation(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    inputDocumentFileLocation#430f0724 id:long access_hash:long version:int = InputFileLocation"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x430f0724

    def __init__(self, id, access_hash, version):
        """
        :param id: Telegram type: «long».
        :param access_hash: Telegram type: «long».
        :param version: Telegram type: «int».

        Constructor for InputFileLocation: Instance of either InputFileLocation, InputEncryptedFileLocation, InputDocumentFileLocation.
        """
        super().__init__()

        self.id = id
        self.access_hash = access_hash
        self.version = version

    def on_send(self, writer):
        writer.write_int(InputDocumentFileLocation.constructor_id, signed=False)
        writer.write_long(self.id)
        writer.write_long(self.access_hash)
        writer.write_int(self.version)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InputDocumentFileLocation(None, None, None)

    def on_response(self, reader):
        self.id = reader.read_long()
        self.access_hash = reader.read_long()
        self.version = reader.read_int()

    def __repr__(self):
        return 'inputDocumentFileLocation#430f0724 id:long access_hash:long version:int = InputFileLocation'

    def __str__(self):
        return '(inputDocumentFileLocation (ID: 0x430f0724) = (id={}, access_hash={}, version={}))'.format(str(self.id), str(self.access_hash), str(self.version))



from ...tl.mtproto_request import MTProtoRequest


class UpdateChatUserTyping(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateChatUserTyping#9a65ea1f chat_id:int user_id:int action:SendMessageAction = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x9a65ea1f

    def __init__(self, chat_id, user_id, action):
        """
        :param chat_id: Telegram type: «int».
        :param user_id: Telegram type: «int».
        :param action: Telegram type: «SendMessageAction».

        Constructor for Update: Instance of either UpdateNewMessage, UpdateMessageID, UpdateDeleteMessages, UpdateUserTyping, UpdateChatUserTyping, UpdateChatParticipants, UpdateUserStatus, UpdateUserName, UpdateUserPhoto, UpdateContactRegistered, UpdateContactLink, UpdateNewEncryptedMessage, UpdateEncryptedChatTyping, UpdateEncryption, UpdateEncryptedMessagesRead, UpdateChatParticipantAdd, UpdateChatParticipantDelete, UpdateDcOptions, UpdateUserBlocked, UpdateNotifySettings, UpdateServiceNotification, UpdatePrivacy, UpdateUserPhone, UpdateReadHistoryInbox, UpdateReadHistoryOutbox, UpdateWebPage, UpdateReadMessagesContents, UpdateChannelTooLong, UpdateChannel, UpdateNewChannelMessage, UpdateReadChannelInbox, UpdateDeleteChannelMessages, UpdateChannelMessageViews, UpdateChatAdmins, UpdateChatParticipantAdmin, UpdateNewStickerSet, UpdateStickerSetsOrder, UpdateStickerSets, UpdateSavedGifs, UpdateBotInlineQuery, UpdateBotInlineSend, UpdateEditChannelMessage, UpdateChannelPinnedMessage, UpdateBotCallbackQuery, UpdateEditMessage, UpdateInlineBotCallbackQuery, UpdateReadChannelOutbox, UpdateDraftMessage, UpdateReadFeaturedStickers, UpdateRecentStickers, UpdateConfig, UpdatePtsChanged, UpdateChannelWebPage, UpdateDialogPinned, UpdatePinnedDialogs, UpdateBotWebhookJSON, UpdateBotWebhookJSONQuery, UpdateBotShippingQuery, UpdateBotPrecheckoutQuery, UpdatePhoneCall.
        """
        super().__init__()

        self.chat_id = chat_id
        self.user_id = user_id
        self.action = action

    def on_send(self, writer):
        writer.write_int(UpdateChatUserTyping.constructor_id, signed=False)
        writer.write_int(self.chat_id)
        writer.write_int(self.user_id)
        self.action.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateChatUserTyping(None, None, None)

    def on_response(self, reader):
        self.chat_id = reader.read_int()
        self.user_id = reader.read_int()
        self.action = reader.tgread_object()

    def __repr__(self):
        return 'updateChatUserTyping#9a65ea1f chat_id:int user_id:int action:SendMessageAction = Update'

    def __str__(self):
        return '(updateChatUserTyping (ID: 0x9a65ea1f) = (chat_id={}, user_id={}, action={}))'.format(str(self.chat_id), str(self.user_id), str(self.action))



from ....tl.mtproto_request import MTProtoRequest


class CdnFile(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    upload.cdnFile#a99fca4f bytes:bytes = upload.CdnFile"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xa99fca4f

    def __init__(self, bytes):
        """
        :param bytes: Telegram type: «bytes».

        Constructor for upload.CdnFile: Instance of either CdnFileReuploadNeeded, CdnFile.
        """
        super().__init__()

        self.bytes = bytes

    def on_send(self, writer):
        writer.write_int(CdnFile.constructor_id, signed=False)
        writer.tgwrite_bytes(self.bytes)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return CdnFile(None)

    def on_response(self, reader):
        self.bytes = reader.tgread_bytes()

    def __repr__(self):
        return 'upload.cdnFile#a99fca4f bytes:bytes = upload.CdnFile'

    def __str__(self):
        return '(upload.cdnFile (ID: 0xa99fca4f) = (bytes={}))'.format(str(self.bytes))



from ...tl.mtproto_request import MTProtoRequest


class LabeledPrice(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    labeledPrice#cb296bf8 label:string amount:long = LabeledPrice"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xcb296bf8

    def __init__(self, label, amount):
        """
        :param label: Telegram type: «string».
        :param amount: Telegram type: «long».

        Constructor for LabeledPrice: Instance of LabeledPrice.
        """
        super().__init__()

        self.label = label
        self.amount = amount

    def on_send(self, writer):
        writer.write_int(LabeledPrice.constructor_id, signed=False)
        writer.tgwrite_string(self.label)
        writer.write_long(self.amount)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return LabeledPrice(None, None)

    def on_response(self, reader):
        self.label = reader.tgread_string()
        self.amount = reader.read_long()

    def __repr__(self):
        return 'labeledPrice#cb296bf8 label:string amount:long = LabeledPrice'

    def __str__(self):
        return '(labeledPrice (ID: 0xcb296bf8) = (label={}, amount={}))'.format(str(self.label), str(self.amount))



from ...tl.mtproto_request import MTProtoRequest


class UpdateChannelWebPage(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateChannelWebPage#40771900 channel_id:int webpage:WebPage pts:int pts_count:int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x40771900

    def __init__(self, channel_id, webpage, pts, pts_count):
        """
        :param channel_id: Telegram type: «int».
        :param webpage: Telegram type: «WebPage».
        :param pts: Telegram type: «int».
        :param pts_count: Telegram type: «int».

        Constructor for Update: Instance of either UpdateNewMessage, UpdateMessageID, UpdateDeleteMessages, UpdateUserTyping, UpdateChatUserTyping, UpdateChatParticipants, UpdateUserStatus, UpdateUserName, UpdateUserPhoto, UpdateContactRegistered, UpdateContactLink, UpdateNewEncryptedMessage, UpdateEncryptedChatTyping, UpdateEncryption, UpdateEncryptedMessagesRead, UpdateChatParticipantAdd, UpdateChatParticipantDelete, UpdateDcOptions, UpdateUserBlocked, UpdateNotifySettings, UpdateServiceNotification, UpdatePrivacy, UpdateUserPhone, UpdateReadHistoryInbox, UpdateReadHistoryOutbox, UpdateWebPage, UpdateReadMessagesContents, UpdateChannelTooLong, UpdateChannel, UpdateNewChannelMessage, UpdateReadChannelInbox, UpdateDeleteChannelMessages, UpdateChannelMessageViews, UpdateChatAdmins, UpdateChatParticipantAdmin, UpdateNewStickerSet, UpdateStickerSetsOrder, UpdateStickerSets, UpdateSavedGifs, UpdateBotInlineQuery, UpdateBotInlineSend, UpdateEditChannelMessage, UpdateChannelPinnedMessage, UpdateBotCallbackQuery, UpdateEditMessage, UpdateInlineBotCallbackQuery, UpdateReadChannelOutbox, UpdateDraftMessage, UpdateReadFeaturedStickers, UpdateRecentStickers, UpdateConfig, UpdatePtsChanged, UpdateChannelWebPage, UpdateDialogPinned, UpdatePinnedDialogs, UpdateBotWebhookJSON, UpdateBotWebhookJSONQuery, UpdateBotShippingQuery, UpdateBotPrecheckoutQuery, UpdatePhoneCall.
        """
        super().__init__()

        self.channel_id = channel_id
        self.webpage = webpage
        self.pts = pts
        self.pts_count = pts_count

    def on_send(self, writer):
        writer.write_int(UpdateChannelWebPage.constructor_id, signed=False)
        writer.write_int(self.channel_id)
        self.webpage.on_send(writer)
        writer.write_int(self.pts)
        writer.write_int(self.pts_count)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateChannelWebPage(None, None, None, None)

    def on_response(self, reader):
        self.channel_id = reader.read_int()
        self.webpage = reader.tgread_object()
        self.pts = reader.read_int()
        self.pts_count = reader.read_int()

    def __repr__(self):
        return 'updateChannelWebPage#40771900 channel_id:int webpage:WebPage pts:int pts_count:int = Update'

    def __str__(self):
        return '(updateChannelWebPage (ID: 0x40771900) = (channel_id={}, webpage={}, pts={}, pts_count={}))'.format(str(self.channel_id), str(self.webpage), str(self.pts), str(self.pts_count))



import re


class ReadCancelledError(Exception):
    """Occurs when a read operation was cancelled"""

    def __init__(self):
        super().__init__(self, 'The read operation was cancelled.')


class InvalidParameterError(Exception):
    """Occurs when an invalid parameter is given, for example,
    when either A or B are required but none is given"""


class TypeNotFoundError(Exception):
    """Occurs when a type is not found, for example,
    when trying to read a TLObject with an invalid constructor code"""

    def __init__(self, invalid_constructor_id):
        super().__init__(
            self, 'Could not find a matching Constructor ID for the TLObject '
            'that was supposed to be read with ID {}. Most likely, a TLObject '
            'was trying to be read when it should not be read.'
            .format(hex(invalid_constructor_id)))

        self.invalid_constructor_id = invalid_constructor_id


class InvalidDCError(Exception):
    def __init__(self, rpc_error):
        self.new_dc = rpc_error.__dict__.pop('additional_data')
        self.__dict__.update(rpc_error.__dict__)


class InvalidChecksumError(Exception):
    def __init__(self, checksum, valid_checksum):
        super().__init__(
            self,
            'Invalid checksum ({} when {} was expected). This packet should be skipped.'
            .format(checksum, valid_checksum))

        self.checksum = checksum
        self.valid_checksum = valid_checksum


class FloodWaitError(Exception):
    def __init__(self, seconds):
        super().__init__(
            self,
            'Too many requests were made too fast. Must wait {} seconds.'
            .format(seconds)
        )
        self.seconds = seconds


class RPCError(Exception):

    CodeMessages = {
        303:
        ('ERROR_SEE_OTHER',
         'The request must be repeated, but directed to a different data center.'
         ),
        400:
        ('BAD_REQUEST',
         'The query contains errors. In the event that a request was created using a '
         'form and contains user generated data, the user should be notified that the '
         'data must be corrected before the query is repeated.'),
        401:
        ('UNAUTHORIZED',
         'There was an unauthorized attempt to use functionality available only to '
         'authorized users.'),
        403:
        ('FORBIDDEN',
         'Privacy violation. For example, an attempt to write a message to someone who '
         'has blacklisted the current user.'),
        404: ('NOT_FOUND',
              'An attempt to invoke a non-existent object, such as a method.'),
        420:
        ('FLOOD',
         'The maximum allowed number of attempts to invoke the given method with '
         'the given input parameters has been exceeded. For example, in an attempt '
         'to request a large number of text messages (SMS) for the same phone number.'
         ),
        500:
        ('INTERNAL',
         'An internal server error occurred while a request was being processed; '
         'for example, there was a disruption while accessing a database or file storage.'
         )
    }

    ErrorMessages = {
        # 303 ERROR_SEE_OTHER
        'FILE_MIGRATE_(\d+)':
        'The file to be accessed is currently stored in a different data center (#{}).',
        'PHONE_MIGRATE_(\d+)':
        'The phone number a user is trying to use for authorization is associated '
        'with a different data center (#{}).',
        'NETWORK_MIGRATE_(\d+)':
        'The source IP address is associated with a different data center (#{}, '
        'for registration).',
        'USER_MIGRATE_(\d+)':
        'The user whose identity is being used to execute queries is associated with '
        'a different data center  (#{} for registration).',

        # 400 BAD_REQUEST
        'FIRSTNAME_INVALID': 'The first name is invalid.',
        'LASTNAME_INVALID': 'The last name is invalid.',
        'PHONE_NUMBER_INVALID': 'The phone number is invalid.',
        'PHONE_CODE_HASH_EMPTY': 'The phone code hash is missing.',
        'PHONE_CODE_EMPTY': 'The phone code is missing.',
        'PHONE_CODE_INVALID': 'The phone code entered was invalid.',
        'PHONE_CODE_EXPIRED': 'The confirmation code has expired.',
        'PHONE_NUMBER_BANNED':
        'The used phone number has been banned from Telegram and cannot '
        'be used anymore. Possibly check https://www.telegram.org/faq_spam.',
        'API_ID_INVALID': 'The api_id/api_hash combination is invalid.',
        'PHONE_NUMBER_OCCUPIED': 'The phone number is already in use.',
        'PHONE_NUMBER_UNOCCUPIED': 'The phone number is not yet being used.',
        'USERS_TOO_FEW': 'Not enough users (to create a chat, for example).',
        'USERS_TOO_MUCH':
        'The maximum number of users has been exceeded (to create a chat, for example).',
        'TYPE_CONSTRUCTOR_INVALID': 'The type constructor is invalid.',
        'FILE_PART_INVALID': 'The file part number is invalid.',
        'FILE_PARTS_INVALID': 'The number of file parts is invalid.',
        'FILE_PART_(\d+)_MISSING':
        'Part {} of the file is missing from storage.',
        'MD5_CHECKSUM_INVALID': 'The MD5 check-sums do not match.',
        'PHOTO_INVALID_DIMENSIONS': 'The photo dimensions are invalid.',
        'FIELD_NAME_INVALID': 'The field with the name FIELD_NAME is invalid.',
        'FIELD_NAME_EMPTY': 'The field with the name FIELD_NAME is missing.',
        'MSG_WAIT_FAILED': 'A waiting call returned an error.',
        'CHAT_ADMIN_REQUIRED':
        'Chat admin privileges are required to do that in the specified chat '
        '(for example, to send a message in a channel which is not yours).',
        'PASSWORD_HASH_INVALID':
        'The password (and thus its hash value) you entered is invalid.',
        'BOT_METHOD_INVALID':
        'The API access for bot users is restricted. The method you tried '
        'to invoke cannot be executed as a bot.',
        'PEER_ID_INVALID':
        'An invalid Peer was used. Make sure to pass the right peer type.',
        'MESSAGE_EMPTY': 'Empty or invalid UTF-8 message was sent.',
        'MESSAGE_TOO_LONG':
        'Message was too long. Current maximum length is 4096 UTF-8 characters.',
        'USERNAME_INVALID':
        'Unacceptable username. Must match r"[a-zA-Z][\w\d]{4,32}"',
        'USERNAME_OCCUPIED': 'The username is already taken.',
        'USERNAME_NOT_OCCUPIED':
        'See issue #96 for Telethon - try upgrading the library.',
        'USERNAME_NOT_MODIFIED':
        'The username is not different from the current username',
        'USER_ID_INVALID':
        'Invalid object ID for an user. Make sure to pass the right types.',
        'CHAT_ID_INVALID':
        'Invalid object ID for a chat. Make sure to pass the right types.',
        'CHANNEL_INVALID':
        'Invalid channel object. Make sure to pass the right types.',
        'MESSAGE_ID_INVALID': 'The specified message ID is invalid.',
        'CONNECTION_LAYER_INVALID':
        'The very first request must always be InvokeWithLayerRequest.',
        'INPUT_METHOD_INVALID':
        'The invoked method does not exist anymore or has never existed.',
        'DC_ID_INVALID':
        'This occurs when an authorization is tried to be exported for '
        'the same data center one is currently connected to.',

        # 401 UNAUTHORIZED
        'AUTH_KEY_UNREGISTERED': 'The key is not registered in the system.',
        'AUTH_KEY_INVALID': 'The key is invalid.',
        'USER_DEACTIVATED': 'The user has been deleted/deactivated.',
        'SESSION_REVOKED':
        'The authorization has been invalidated, because of the user terminating all sessions.',
        'SESSION_EXPIRED': 'The authorization has expired.',
        'ACTIVE_USER_REQUIRED':
        'The method is only available to already activated users.',
        'AUTH_KEY_PERM_EMPTY':
        'The method is unavailable for temporary authorization key, not bound to permanent.',
        'SESSION_PASSWORD_NEEDED':
        'Two-steps verification is enabled and a password is required.',

        # 420 FLOOD
        'FLOOD_WAIT_(\d+)': 'A wait of {} seconds is required.'
    }

    def __init__(self, code, message):
        self.code = code
        self.code_meaning = RPCError.CodeMessages[code]

        self.message = message
        self.must_resend = code == 303  # ERROR_SEE_OTHER, "The request must be repeated"

        called_super = False
        for key, error_msg in RPCError.ErrorMessages.items():
            match = re.match(key, message)
            if match:
                error_msg = '{} ({}): {}'.format(
                    self.message, self.code, error_msg)

                # Get additional_data, if any
                if match.groups():
                    self.additional_data = int(match.group(1))
                    super().__init__(self,
                                     error_msg.format(self.additional_data))
                else:
                    self.additional_data = None
                    super().__init__(self, error_msg)

                    # Add another field to easily determine whether this error
                    # should be handled as a password-required error
                    self.password_required = message == 'SESSION_PASSWORD_NEEDED'

                called_super = True
                break

        if not called_super:
            super().__init__(
                self, 'Unknown error message with code {}: {}'.format(code,
                                                                      message))


class BadMessageError(Exception):
    """Occurs when handling a bad_message_notification"""
    ErrorMessages = {
        16:
        'msg_id too low (most likely, client time is wrong it would be worthwhile to '
        'synchronize it using msg_id notifications and re-send the original message '
        'with the "correct" msg_id or wrap it in a container with a new msg_id if the '
        'original message had waited too long on the client to be transmitted).',
        17:
        'msg_id too high (similar to the previous case, the client time has to be '
        'synchronized, and the message re-sent with the correct msg_id).',
        18:
        'Incorrect two lower order msg_id bits (the server expects client message msg_id '
        'to be divisible by 4).',
        19:
        'Container msg_id is the same as msg_id of a previously received message '
        '(this must never happen).',
        20:
        'Message too old, and it cannot be verified whether the server has received a '
        'message with this msg_id or not.',
        32:
        'msg_seqno too low (the server has already received a message with a lower '
        'msg_id but with either a higher or an equal and odd seqno).',
        33:
        'msg_seqno too high (similarly, there is a message with a higher msg_id but with '
        'either a lower or an equal and odd seqno).',
        34:
        'An even msg_seqno expected (irrelevant message), but odd received.',
        35: 'Odd msg_seqno expected (relevant message), but even received.',
        48:
        'Incorrect server salt (in this case, the bad_server_salt response is received with '
        'the correct salt, and the message is to be re-sent with it).',
        64: 'Invalid container.'
    }

    def __init__(self, code):
        super().__init__(self, BadMessageError.ErrorMessages.get(
            code,
            'Unknown error code (this should not happen): {}.'.format(code)))

        self.code = code



from ...tl.mtproto_request import MTProtoRequest


class PhoneCallDiscardReasonMissed(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    phoneCallDiscardReasonMissed#85e42301  = PhoneCallDiscardReason"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x85e42301

    def __init__(self):
        super().__init__()

    def on_send(self, writer):
        writer.write_int(PhoneCallDiscardReasonMissed.constructor_id, signed=False)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return PhoneCallDiscardReasonMissed()

    def on_response(self, reader):
        pass

    def __repr__(self):
        return 'phoneCallDiscardReasonMissed#85e42301  = PhoneCallDiscardReason'

    def __str__(self):
        return '(phoneCallDiscardReasonMissed (ID: 0x85e42301) = ())'.format()



from ...tl.mtproto_request import MTProtoRequest


class Pong(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    pong#347773c5 msg_id:long ping_id:long = Pong"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x347773c5

    def __init__(self, msg_id, ping_id):
        """
        :param msg_id: Telegram type: «long».
        :param ping_id: Telegram type: «long».

        Constructor for Pong: Instance of Pong.
        """
        super().__init__()

        self.msg_id = msg_id
        self.ping_id = ping_id

    def on_send(self, writer):
        writer.write_int(Pong.constructor_id, signed=False)
        writer.write_long(self.msg_id)
        writer.write_long(self.ping_id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return Pong(None, None)

    def on_response(self, reader):
        self.msg_id = reader.read_long()
        self.ping_id = reader.read_long()

    def __repr__(self):
        return 'pong#347773c5 msg_id:long ping_id:long = Pong'

    def __str__(self):
        return '(pong (ID: 0x347773c5) = (msg_id={}, ping_id={}))'.format(str(self.msg_id), str(self.ping_id))



from ...tl.mtproto_request import MTProtoRequest


class ChatParticipantsForbidden(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    chatParticipantsForbidden#fc900c2b flags:# chat_id:int self_participant:flags.0?ChatParticipant = ChatParticipants"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xfc900c2b

    def __init__(self, chat_id, self_participant=None):
        """
        :param chat_id: Telegram type: «int».
        :param self_participant: Telegram type: «ChatParticipant».

        Constructor for ChatParticipants: Instance of either ChatParticipantsForbidden, ChatParticipants.
        """
        super().__init__()

        self.chat_id = chat_id
        self.self_participant = self_participant

    def on_send(self, writer):
        writer.write_int(ChatParticipantsForbidden.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.self_participant else 0
        writer.write_int(flags)

        writer.write_int(self.chat_id)
        if self.self_participant:
            self.self_participant.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ChatParticipantsForbidden(None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        self.chat_id = reader.read_int()
        if (flags & (1 << 0)) != 0:
            self.self_participant = reader.tgread_object()

    def __repr__(self):
        return 'chatParticipantsForbidden#fc900c2b flags:# chat_id:int self_participant:flags.0?ChatParticipant = ChatParticipants'

    def __str__(self):
        return '(chatParticipantsForbidden (ID: 0xfc900c2b) = (chat_id={}, self_participant={}))'.format(str(self.chat_id), str(self.self_participant))



from ...tl.mtproto_request import MTProtoRequest


class CdnPublicKey(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    cdnPublicKey#c982eaba dc_id:int public_key:string = CdnPublicKey"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xc982eaba

    def __init__(self, dc_id, public_key):
        """
        :param dc_id: Telegram type: «int».
        :param public_key: Telegram type: «string».

        Constructor for CdnPublicKey: Instance of CdnPublicKey.
        """
        super().__init__()

        self.dc_id = dc_id
        self.public_key = public_key

    def on_send(self, writer):
        writer.write_int(CdnPublicKey.constructor_id, signed=False)
        writer.write_int(self.dc_id)
        writer.tgwrite_string(self.public_key)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return CdnPublicKey(None, None)

    def on_response(self, reader):
        self.dc_id = reader.read_int()
        self.public_key = reader.tgread_string()

    def __repr__(self):
        return 'cdnPublicKey#c982eaba dc_id:int public_key:string = CdnPublicKey'

    def __str__(self):
        return '(cdnPublicKey (ID: 0xc982eaba) = (dc_id={}, public_key={}))'.format(str(self.dc_id), str(self.public_key))



from ...tl.mtproto_request import MTProtoRequest


class PageBlockEmbed(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    pageBlockEmbed#cde200d1 flags:# full_width:flags.0?true allow_scrolling:flags.3?true url:flags.1?string html:flags.2?string poster_photo_id:flags.4?long w:int h:int caption:RichText = PageBlock"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xcde200d1

    def __init__(self, w, h, caption, full_width=None, allow_scrolling=None, url=None, html=None, poster_photo_id=None):
        """
        :param full_width: Telegram type: «true».
        :param allow_scrolling: Telegram type: «true».
        :param url: Telegram type: «string».
        :param html: Telegram type: «string».
        :param poster_photo_id: Telegram type: «long».
        :param w: Telegram type: «int».
        :param h: Telegram type: «int».
        :param caption: Telegram type: «RichText».

        Constructor for PageBlock: Instance of either PageBlockUnsupported, PageBlockTitle, PageBlockSubtitle, PageBlockAuthorDate, PageBlockHeader, PageBlockSubheader, PageBlockParagraph, PageBlockPreformatted, PageBlockFooter, PageBlockDivider, PageBlockAnchor, PageBlockList, PageBlockBlockquote, PageBlockPullquote, PageBlockPhoto, PageBlockVideo, PageBlockCover, PageBlockEmbed, PageBlockEmbedPost, PageBlockCollage, PageBlockSlideshow, PageBlockChannel.
        """
        super().__init__()

        self.full_width = full_width
        self.allow_scrolling = allow_scrolling
        self.url = url
        self.html = html
        self.poster_photo_id = poster_photo_id
        self.w = w
        self.h = h
        self.caption = caption

    def on_send(self, writer):
        writer.write_int(PageBlockEmbed.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.full_width else 0
        flags |= (1 << 3) if self.allow_scrolling else 0
        flags |= (1 << 1) if self.url else 0
        flags |= (1 << 2) if self.html else 0
        flags |= (1 << 4) if self.poster_photo_id else 0
        writer.write_int(flags)

        if self.url:
            writer.tgwrite_string(self.url)

        if self.html:
            writer.tgwrite_string(self.html)

        if self.poster_photo_id:
            writer.write_long(self.poster_photo_id)

        writer.write_int(self.w)
        writer.write_int(self.h)
        self.caption.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return PageBlockEmbed(None, None, None, None, None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 0)) != 0:
            self.full_width = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 3)) != 0:
            self.allow_scrolling = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 1)) != 0:
            self.url = reader.tgread_string()

        if (flags & (1 << 2)) != 0:
            self.html = reader.tgread_string()

        if (flags & (1 << 4)) != 0:
            self.poster_photo_id = reader.read_long()

        self.w = reader.read_int()
        self.h = reader.read_int()
        self.caption = reader.tgread_object()

    def __repr__(self):
        return 'pageBlockEmbed#cde200d1 flags:# full_width:flags.0?true allow_scrolling:flags.3?true url:flags.1?string html:flags.2?string poster_photo_id:flags.4?long w:int h:int caption:RichText = PageBlock'

    def __str__(self):
        return '(pageBlockEmbed (ID: 0xcde200d1) = (full_width={}, allow_scrolling={}, url={}, html={}, poster_photo_id={}, w={}, h={}, caption={}))'.format(str(self.full_width), str(self.allow_scrolling), str(self.url), str(self.html), str(self.poster_photo_id), str(self.w), str(self.h), str(self.caption))



from ...tl.mtproto_request import MTProtoRequest


class UpdateReadMessagesContents(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateReadMessagesContents#68c13933 messages:Vector<int> pts:int pts_count:int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x68c13933

    def __init__(self, messages, pts, pts_count):
        """
        :param messages: Telegram type: «int». Must be a list.
        :param pts: Telegram type: «int».
        :param pts_count: Telegram type: «int».

        Constructor for Update: Instance of either UpdateNewMessage, UpdateMessageID, UpdateDeleteMessages, UpdateUserTyping, UpdateChatUserTyping, UpdateChatParticipants, UpdateUserStatus, UpdateUserName, UpdateUserPhoto, UpdateContactRegistered, UpdateContactLink, UpdateNewEncryptedMessage, UpdateEncryptedChatTyping, UpdateEncryption, UpdateEncryptedMessagesRead, UpdateChatParticipantAdd, UpdateChatParticipantDelete, UpdateDcOptions, UpdateUserBlocked, UpdateNotifySettings, UpdateServiceNotification, UpdatePrivacy, UpdateUserPhone, UpdateReadHistoryInbox, UpdateReadHistoryOutbox, UpdateWebPage, UpdateReadMessagesContents, UpdateChannelTooLong, UpdateChannel, UpdateNewChannelMessage, UpdateReadChannelInbox, UpdateDeleteChannelMessages, UpdateChannelMessageViews, UpdateChatAdmins, UpdateChatParticipantAdmin, UpdateNewStickerSet, UpdateStickerSetsOrder, UpdateStickerSets, UpdateSavedGifs, UpdateBotInlineQuery, UpdateBotInlineSend, UpdateEditChannelMessage, UpdateChannelPinnedMessage, UpdateBotCallbackQuery, UpdateEditMessage, UpdateInlineBotCallbackQuery, UpdateReadChannelOutbox, UpdateDraftMessage, UpdateReadFeaturedStickers, UpdateRecentStickers, UpdateConfig, UpdatePtsChanged, UpdateChannelWebPage, UpdateDialogPinned, UpdatePinnedDialogs, UpdateBotWebhookJSON, UpdateBotWebhookJSONQuery, UpdateBotShippingQuery, UpdateBotPrecheckoutQuery, UpdatePhoneCall.
        """
        super().__init__()

        self.messages = messages
        self.pts = pts
        self.pts_count = pts_count

    def on_send(self, writer):
        writer.write_int(UpdateReadMessagesContents.constructor_id, signed=False)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.messages))
        for messages_item in self.messages:
            writer.write_int(messages_item)

        writer.write_int(self.pts)
        writer.write_int(self.pts_count)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateReadMessagesContents(None, None, None)

    def on_response(self, reader):
        reader.read_int()  # Vector's constructor ID
        self.messages = []  # Initialize an empty list
        messages_len = reader.read_int()
        for _ in range(messages_len):
            messages_item = reader.read_int()
            self.messages.append(messages_item)

        self.pts = reader.read_int()
        self.pts_count = reader.read_int()

    def __repr__(self):
        return 'updateReadMessagesContents#68c13933 messages:Vector<int> pts:int pts_count:int = Update'

    def __str__(self):
        return '(updateReadMessagesContents (ID: 0x68c13933) = (messages={}, pts={}, pts_count={}))'.format(None if not self.messages else [str(_) for _ in self.messages], str(self.pts), str(self.pts_count))



from ...tl.mtproto_request import MTProtoRequest


class UpdateEditMessage(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateEditMessage#e40370a3 message:Message pts:int pts_count:int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xe40370a3

    def __init__(self, message, pts, pts_count):
        """
        :param message: Telegram type: «Message».
        :param pts: Telegram type: «int».
        :param pts_count: Telegram type: «int».

        Constructor for Update: Instance of either UpdateNewMessage, UpdateMessageID, UpdateDeleteMessages, UpdateUserTyping, UpdateChatUserTyping, UpdateChatParticipants, UpdateUserStatus, UpdateUserName, UpdateUserPhoto, UpdateContactRegistered, UpdateContactLink, UpdateNewEncryptedMessage, UpdateEncryptedChatTyping, UpdateEncryption, UpdateEncryptedMessagesRead, UpdateChatParticipantAdd, UpdateChatParticipantDelete, UpdateDcOptions, UpdateUserBlocked, UpdateNotifySettings, UpdateServiceNotification, UpdatePrivacy, UpdateUserPhone, UpdateReadHistoryInbox, UpdateReadHistoryOutbox, UpdateWebPage, UpdateReadMessagesContents, UpdateChannelTooLong, UpdateChannel, UpdateNewChannelMessage, UpdateReadChannelInbox, UpdateDeleteChannelMessages, UpdateChannelMessageViews, UpdateChatAdmins, UpdateChatParticipantAdmin, UpdateNewStickerSet, UpdateStickerSetsOrder, UpdateStickerSets, UpdateSavedGifs, UpdateBotInlineQuery, UpdateBotInlineSend, UpdateEditChannelMessage, UpdateChannelPinnedMessage, UpdateBotCallbackQuery, UpdateEditMessage, UpdateInlineBotCallbackQuery, UpdateReadChannelOutbox, UpdateDraftMessage, UpdateReadFeaturedStickers, UpdateRecentStickers, UpdateConfig, UpdatePtsChanged, UpdateChannelWebPage, UpdateDialogPinned, UpdatePinnedDialogs, UpdateBotWebhookJSON, UpdateBotWebhookJSONQuery, UpdateBotShippingQuery, UpdateBotPrecheckoutQuery, UpdatePhoneCall.
        """
        super().__init__()

        self.message = message
        self.pts = pts
        self.pts_count = pts_count

    def on_send(self, writer):
        writer.write_int(UpdateEditMessage.constructor_id, signed=False)
        self.message.on_send(writer)
        writer.write_int(self.pts)
        writer.write_int(self.pts_count)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateEditMessage(None, None, None)

    def on_response(self, reader):
        self.message = reader.tgread_object()
        self.pts = reader.read_int()
        self.pts_count = reader.read_int()

    def __repr__(self):
        return 'updateEditMessage#e40370a3 message:Message pts:int pts_count:int = Update'

    def __str__(self):
        return '(updateEditMessage (ID: 0xe40370a3) = (message={}, pts={}, pts_count={}))'.format(str(self.message), str(self.pts), str(self.pts_count))



from ....tl.mtproto_request import MTProtoRequest


class ExportCardRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    contacts.exportCard#84e53737  = Vector<int>"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x84e53737

    def __init__(self):
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

    def on_send(self, writer):
        writer.write_int(ExportCardRequest.constructor_id, signed=False)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ExportCardRequest()

    def on_response(self, reader):
        reader.read_int()  # Vector id
        count = reader.read_int()
        self.result = [reader.read_int() for _ in range(count)]

    def __repr__(self):
        return 'contacts.exportCard#84e53737  = Vector<int>'

    def __str__(self):
        return '(contacts.exportCard (ID: 0x84e53737) = ())'.format()



from ....tl.mtproto_request import MTProtoRequest


class SendEncryptedServiceRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.sendEncryptedService#32d439a4 peer:InputEncryptedChat random_id:long data:bytes = messages.SentEncryptedMessage"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x32d439a4

    def __init__(self, peer, random_id, data):
        """
        :param peer: Telegram type: «InputEncryptedChat».
        :param random_id: Telegram type: «long».
        :param data: Telegram type: «bytes».

        :returns messages.SentEncryptedMessage: Instance of either SentEncryptedMessage, SentEncryptedFile.
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.peer = peer
        self.random_id = random_id
        self.data = data

    def on_send(self, writer):
        writer.write_int(SendEncryptedServiceRequest.constructor_id, signed=False)
        self.peer.on_send(writer)
        writer.write_long(self.random_id)
        writer.tgwrite_bytes(self.data)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return SendEncryptedServiceRequest(None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.sendEncryptedService#32d439a4 peer:InputEncryptedChat random_id:long data:bytes = messages.SentEncryptedMessage'

    def __str__(self):
        return '(messages.sendEncryptedService (ID: 0x32d439a4) = (peer={}, random_id={}, data={}))'.format(str(self.peer), str(self.random_id), str(self.data))



from ...tl.mtproto_request import MTProtoRequest


class UpdateBotShippingQuery(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateBotShippingQuery#e0cdc940 query_id:long user_id:int payload:bytes shipping_address:PostAddress = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xe0cdc940

    def __init__(self, query_id, user_id, payload, shipping_address):
        """
        :param query_id: Telegram type: «long».
        :param user_id: Telegram type: «int».
        :param payload: Telegram type: «bytes».
        :param shipping_address: Telegram type: «PostAddress».

        Constructor for Update: Instance of either UpdateNewMessage, UpdateMessageID, UpdateDeleteMessages, UpdateUserTyping, UpdateChatUserTyping, UpdateChatParticipants, UpdateUserStatus, UpdateUserName, UpdateUserPhoto, UpdateContactRegistered, UpdateContactLink, UpdateNewEncryptedMessage, UpdateEncryptedChatTyping, UpdateEncryption, UpdateEncryptedMessagesRead, UpdateChatParticipantAdd, UpdateChatParticipantDelete, UpdateDcOptions, UpdateUserBlocked, UpdateNotifySettings, UpdateServiceNotification, UpdatePrivacy, UpdateUserPhone, UpdateReadHistoryInbox, UpdateReadHistoryOutbox, UpdateWebPage, UpdateReadMessagesContents, UpdateChannelTooLong, UpdateChannel, UpdateNewChannelMessage, UpdateReadChannelInbox, UpdateDeleteChannelMessages, UpdateChannelMessageViews, UpdateChatAdmins, UpdateChatParticipantAdmin, UpdateNewStickerSet, UpdateStickerSetsOrder, UpdateStickerSets, UpdateSavedGifs, UpdateBotInlineQuery, UpdateBotInlineSend, UpdateEditChannelMessage, UpdateChannelPinnedMessage, UpdateBotCallbackQuery, UpdateEditMessage, UpdateInlineBotCallbackQuery, UpdateReadChannelOutbox, UpdateDraftMessage, UpdateReadFeaturedStickers, UpdateRecentStickers, UpdateConfig, UpdatePtsChanged, UpdateChannelWebPage, UpdateDialogPinned, UpdatePinnedDialogs, UpdateBotWebhookJSON, UpdateBotWebhookJSONQuery, UpdateBotShippingQuery, UpdateBotPrecheckoutQuery, UpdatePhoneCall.
        """
        super().__init__()

        self.query_id = query_id
        self.user_id = user_id
        self.payload = payload
        self.shipping_address = shipping_address

    def on_send(self, writer):
        writer.write_int(UpdateBotShippingQuery.constructor_id, signed=False)
        writer.write_long(self.query_id)
        writer.write_int(self.user_id)
        writer.tgwrite_bytes(self.payload)
        self.shipping_address.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateBotShippingQuery(None, None, None, None)

    def on_response(self, reader):
        self.query_id = reader.read_long()
        self.user_id = reader.read_int()
        self.payload = reader.tgread_bytes()
        self.shipping_address = reader.tgread_object()

    def __repr__(self):
        return 'updateBotShippingQuery#e0cdc940 query_id:long user_id:int payload:bytes shipping_address:PostAddress = Update'

    def __str__(self):
        return '(updateBotShippingQuery (ID: 0xe0cdc940) = (query_id={}, user_id={}, payload={}, shipping_address={}))'.format(str(self.query_id), str(self.user_id), str(self.payload), str(self.shipping_address))



from ....tl.mtproto_request import MTProtoRequest


class SetBotUpdatesStatusRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    help.setBotUpdatesStatus#ec22cfcd pending_updates_count:int message:string = Bool"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xec22cfcd

    def __init__(self, pending_updates_count, message):
        """
        :param pending_updates_count: Telegram type: «int».
        :param message: Telegram type: «string».

        :returns Bool: This type has no constructors.
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.pending_updates_count = pending_updates_count
        self.message = message

    def on_send(self, writer):
        writer.write_int(SetBotUpdatesStatusRequest.constructor_id, signed=False)
        writer.write_int(self.pending_updates_count)
        writer.tgwrite_string(self.message)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return SetBotUpdatesStatusRequest(None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'help.setBotUpdatesStatus#ec22cfcd pending_updates_count:int message:string = Bool'

    def __str__(self):
        return '(help.setBotUpdatesStatus (ID: 0xec22cfcd) = (pending_updates_count={}, message={}))'.format(str(self.pending_updates_count), str(self.message))



from ...tl.mtproto_request import MTProtoRequest


class UpdateChatParticipantDelete(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateChatParticipantDelete#6e5f8c22 chat_id:int user_id:int version:int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x6e5f8c22

    def __init__(self, chat_id, user_id, version):
        """
        :param chat_id: Telegram type: «int».
        :param user_id: Telegram type: «int».
        :param version: Telegram type: «int».

        Constructor for Update: Instance of either UpdateNewMessage, UpdateMessageID, UpdateDeleteMessages, UpdateUserTyping, UpdateChatUserTyping, UpdateChatParticipants, UpdateUserStatus, UpdateUserName, UpdateUserPhoto, UpdateContactRegistered, UpdateContactLink, UpdateNewEncryptedMessage, UpdateEncryptedChatTyping, UpdateEncryption, UpdateEncryptedMessagesRead, UpdateChatParticipantAdd, UpdateChatParticipantDelete, UpdateDcOptions, UpdateUserBlocked, UpdateNotifySettings, UpdateServiceNotification, UpdatePrivacy, UpdateUserPhone, UpdateReadHistoryInbox, UpdateReadHistoryOutbox, UpdateWebPage, UpdateReadMessagesContents, UpdateChannelTooLong, UpdateChannel, UpdateNewChannelMessage, UpdateReadChannelInbox, UpdateDeleteChannelMessages, UpdateChannelMessageViews, UpdateChatAdmins, UpdateChatParticipantAdmin, UpdateNewStickerSet, UpdateStickerSetsOrder, UpdateStickerSets, UpdateSavedGifs, UpdateBotInlineQuery, UpdateBotInlineSend, UpdateEditChannelMessage, UpdateChannelPinnedMessage, UpdateBotCallbackQuery, UpdateEditMessage, UpdateInlineBotCallbackQuery, UpdateReadChannelOutbox, UpdateDraftMessage, UpdateReadFeaturedStickers, UpdateRecentStickers, UpdateConfig, UpdatePtsChanged, UpdateChannelWebPage, UpdateDialogPinned, UpdatePinnedDialogs, UpdateBotWebhookJSON, UpdateBotWebhookJSONQuery, UpdateBotShippingQuery, UpdateBotPrecheckoutQuery, UpdatePhoneCall.
        """
        super().__init__()

        self.chat_id = chat_id
        self.user_id = user_id
        self.version = version

    def on_send(self, writer):
        writer.write_int(UpdateChatParticipantDelete.constructor_id, signed=False)
        writer.write_int(self.chat_id)
        writer.write_int(self.user_id)
        writer.write_int(self.version)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateChatParticipantDelete(None, None, None)

    def on_response(self, reader):
        self.chat_id = reader.read_int()
        self.user_id = reader.read_int()
        self.version = reader.read_int()

    def __repr__(self):
        return 'updateChatParticipantDelete#6e5f8c22 chat_id:int user_id:int version:int = Update'

    def __str__(self):
        return '(updateChatParticipantDelete (ID: 0x6e5f8c22) = (chat_id={}, user_id={}, version={}))'.format(str(self.chat_id), str(self.user_id), str(self.version))



from ...tl.mtproto_request import MTProtoRequest


class UpdatePrivacy(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updatePrivacy#ee3b272a key:PrivacyKey rules:Vector<PrivacyRule> = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xee3b272a

    def __init__(self, key, rules):
        """
        :param key: Telegram type: «PrivacyKey».
        :param rules: Telegram type: «PrivacyRule». Must be a list.

        Constructor for Update: Instance of either UpdateNewMessage, UpdateMessageID, UpdateDeleteMessages, UpdateUserTyping, UpdateChatUserTyping, UpdateChatParticipants, UpdateUserStatus, UpdateUserName, UpdateUserPhoto, UpdateContactRegistered, UpdateContactLink, UpdateNewEncryptedMessage, UpdateEncryptedChatTyping, UpdateEncryption, UpdateEncryptedMessagesRead, UpdateChatParticipantAdd, UpdateChatParticipantDelete, UpdateDcOptions, UpdateUserBlocked, UpdateNotifySettings, UpdateServiceNotification, UpdatePrivacy, UpdateUserPhone, UpdateReadHistoryInbox, UpdateReadHistoryOutbox, UpdateWebPage, UpdateReadMessagesContents, UpdateChannelTooLong, UpdateChannel, UpdateNewChannelMessage, UpdateReadChannelInbox, UpdateDeleteChannelMessages, UpdateChannelMessageViews, UpdateChatAdmins, UpdateChatParticipantAdmin, UpdateNewStickerSet, UpdateStickerSetsOrder, UpdateStickerSets, UpdateSavedGifs, UpdateBotInlineQuery, UpdateBotInlineSend, UpdateEditChannelMessage, UpdateChannelPinnedMessage, UpdateBotCallbackQuery, UpdateEditMessage, UpdateInlineBotCallbackQuery, UpdateReadChannelOutbox, UpdateDraftMessage, UpdateReadFeaturedStickers, UpdateRecentStickers, UpdateConfig, UpdatePtsChanged, UpdateChannelWebPage, UpdateDialogPinned, UpdatePinnedDialogs, UpdateBotWebhookJSON, UpdateBotWebhookJSONQuery, UpdateBotShippingQuery, UpdateBotPrecheckoutQuery, UpdatePhoneCall.
        """
        super().__init__()

        self.key = key
        self.rules = rules

    def on_send(self, writer):
        writer.write_int(UpdatePrivacy.constructor_id, signed=False)
        self.key.on_send(writer)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.rules))
        for rules_item in self.rules:
            rules_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdatePrivacy(None, None)

    def on_response(self, reader):
        self.key = reader.tgread_object()
        reader.read_int()  # Vector's constructor ID
        self.rules = []  # Initialize an empty list
        rules_len = reader.read_int()
        for _ in range(rules_len):
            rules_item = reader.tgread_object()
            self.rules.append(rules_item)

    def __repr__(self):
        return 'updatePrivacy#ee3b272a key:PrivacyKey rules:Vector<PrivacyRule> = Update'

    def __str__(self):
        return '(updatePrivacy (ID: 0xee3b272a) = (key={}, rules={}))'.format(str(self.key), None if not self.rules else [str(_) for _ in self.rules])



from ...tl.mtproto_request import MTProtoRequest


class UpdateUserName(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateUserName#a7332b73 user_id:int first_name:string last_name:string username:string = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xa7332b73

    def __init__(self, user_id, first_name, last_name, username):
        """
        :param user_id: Telegram type: «int».
        :param first_name: Telegram type: «string».
        :param last_name: Telegram type: «string».
        :param username: Telegram type: «string».

        Constructor for Update: Instance of either UpdateNewMessage, UpdateMessageID, UpdateDeleteMessages, UpdateUserTyping, UpdateChatUserTyping, UpdateChatParticipants, UpdateUserStatus, UpdateUserName, UpdateUserPhoto, UpdateContactRegistered, UpdateContactLink, UpdateNewEncryptedMessage, UpdateEncryptedChatTyping, UpdateEncryption, UpdateEncryptedMessagesRead, UpdateChatParticipantAdd, UpdateChatParticipantDelete, UpdateDcOptions, UpdateUserBlocked, UpdateNotifySettings, UpdateServiceNotification, UpdatePrivacy, UpdateUserPhone, UpdateReadHistoryInbox, UpdateReadHistoryOutbox, UpdateWebPage, UpdateReadMessagesContents, UpdateChannelTooLong, UpdateChannel, UpdateNewChannelMessage, UpdateReadChannelInbox, UpdateDeleteChannelMessages, UpdateChannelMessageViews, UpdateChatAdmins, UpdateChatParticipantAdmin, UpdateNewStickerSet, UpdateStickerSetsOrder, UpdateStickerSets, UpdateSavedGifs, UpdateBotInlineQuery, UpdateBotInlineSend, UpdateEditChannelMessage, UpdateChannelPinnedMessage, UpdateBotCallbackQuery, UpdateEditMessage, UpdateInlineBotCallbackQuery, UpdateReadChannelOutbox, UpdateDraftMessage, UpdateReadFeaturedStickers, UpdateRecentStickers, UpdateConfig, UpdatePtsChanged, UpdateChannelWebPage, UpdateDialogPinned, UpdatePinnedDialogs, UpdateBotWebhookJSON, UpdateBotWebhookJSONQuery, UpdateBotShippingQuery, UpdateBotPrecheckoutQuery, UpdatePhoneCall.
        """
        super().__init__()

        self.user_id = user_id
        self.first_name = first_name
        self.last_name = last_name
        self.username = username

    def on_send(self, writer):
        writer.write_int(UpdateUserName.constructor_id, signed=False)
        writer.write_int(self.user_id)
        writer.tgwrite_string(self.first_name)
        writer.tgwrite_string(self.last_name)
        writer.tgwrite_string(self.username)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateUserName(None, None, None, None)

    def on_response(self, reader):
        self.user_id = reader.read_int()
        self.first_name = reader.tgread_string()
        self.last_name = reader.tgread_string()
        self.username = reader.tgread_string()

    def __repr__(self):
        return 'updateUserName#a7332b73 user_id:int first_name:string last_name:string username:string = Update'

    def __str__(self):
        return '(updateUserName (ID: 0xa7332b73) = (user_id={}, first_name={}, last_name={}, username={}))'.format(str(self.user_id), str(self.first_name), str(self.last_name), str(self.username))



from ...tl.mtproto_request import MTProtoRequest


class ChatFull(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    chatFull#2e02a614 id:int participants:ChatParticipants chat_photo:Photo notify_settings:PeerNotifySettings exported_invite:ExportedChatInvite bot_info:Vector<BotInfo> = ChatFull"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x2e02a614

    def __init__(self, id, participants, chat_photo, notify_settings, exported_invite, bot_info):
        """
        :param id: Telegram type: «int».
        :param participants: Telegram type: «ChatParticipants».
        :param chat_photo: Telegram type: «Photo».
        :param notify_settings: Telegram type: «PeerNotifySettings».
        :param exported_invite: Telegram type: «ExportedChatInvite».
        :param bot_info: Telegram type: «BotInfo». Must be a list.

        Constructor for ChatFull: Instance of either ChatFull, ChannelFull.
        """
        super().__init__()

        self.id = id
        self.participants = participants
        self.chat_photo = chat_photo
        self.notify_settings = notify_settings
        self.exported_invite = exported_invite
        self.bot_info = bot_info

    def on_send(self, writer):
        writer.write_int(ChatFull.constructor_id, signed=False)
        writer.write_int(self.id)
        self.participants.on_send(writer)
        self.chat_photo.on_send(writer)
        self.notify_settings.on_send(writer)
        self.exported_invite.on_send(writer)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.bot_info))
        for bot_info_item in self.bot_info:
            bot_info_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ChatFull(None, None, None, None, None, None)

    def on_response(self, reader):
        self.id = reader.read_int()
        self.participants = reader.tgread_object()
        self.chat_photo = reader.tgread_object()
        self.notify_settings = reader.tgread_object()
        self.exported_invite = reader.tgread_object()
        reader.read_int()  # Vector's constructor ID
        self.bot_info = []  # Initialize an empty list
        bot_info_len = reader.read_int()
        for _ in range(bot_info_len):
            bot_info_item = reader.tgread_object()
            self.bot_info.append(bot_info_item)

    def __repr__(self):
        return 'chatFull#2e02a614 id:int participants:ChatParticipants chat_photo:Photo notify_settings:PeerNotifySettings exported_invite:ExportedChatInvite bot_info:Vector<BotInfo> = ChatFull'

    def __str__(self):
        return '(chatFull (ID: 0x2e02a614) = (id={}, participants={}, chat_photo={}, notify_settings={}, exported_invite={}, bot_info={}))'.format(str(self.id), str(self.participants), str(self.chat_photo), str(self.notify_settings), str(self.exported_invite), None if not self.bot_info else [str(_) for _ in self.bot_info])



from ....tl.mtproto_request import MTProtoRequest


class GetBotCallbackAnswerRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.getBotCallbackAnswer#810a9fec flags:# game:flags.1?true peer:InputPeer msg_id:int data:flags.0?bytes = messages.BotCallbackAnswer"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x810a9fec

    def __init__(self, peer, msg_id, game=None, data=None):
        """
        :param game: Telegram type: «true».
        :param peer: Telegram type: «InputPeer».
        :param msg_id: Telegram type: «int».
        :param data: Telegram type: «bytes».

        :returns messages.BotCallbackAnswer: Instance of BotCallbackAnswer.
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.game = game
        self.peer = peer
        self.msg_id = msg_id
        self.data = data

    def on_send(self, writer):
        writer.write_int(GetBotCallbackAnswerRequest.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 1) if self.game else 0
        flags |= (1 << 0) if self.data else 0
        writer.write_int(flags)

        self.peer.on_send(writer)
        writer.write_int(self.msg_id)
        if self.data:
            writer.tgwrite_bytes(self.data)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return GetBotCallbackAnswerRequest(None, None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.getBotCallbackAnswer#810a9fec flags:# game:flags.1?true peer:InputPeer msg_id:int data:flags.0?bytes = messages.BotCallbackAnswer'

    def __str__(self):
        return '(messages.getBotCallbackAnswer (ID: 0x810a9fec) = (game={}, peer={}, msg_id={}, data={}))'.format(str(self.game), str(self.peer), str(self.msg_id), str(self.data))



from ....tl.mtproto_request import MTProtoRequest


class GetPaymentFormRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    payments.getPaymentForm#99f09745 msg_id:int = payments.PaymentForm"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x99f09745

    def __init__(self, msg_id):
        """
        :param msg_id: Telegram type: «int».

        :returns payments.PaymentForm: Instance of PaymentForm.
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.msg_id = msg_id

    def on_send(self, writer):
        writer.write_int(GetPaymentFormRequest.constructor_id, signed=False)
        writer.write_int(self.msg_id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return GetPaymentFormRequest(None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'payments.getPaymentForm#99f09745 msg_id:int = payments.PaymentForm'

    def __str__(self):
        return '(payments.getPaymentForm (ID: 0x99f09745) = (msg_id={}))'.format(str(self.msg_id))



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# To use a consistent encoding
from codecs import open
from os import path

# Always prefer setuptools over distutils
from setuptools import find_packages, setup

from telethon import TelegramClient

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='Telethon',

    # Versions should comply with PEP440.
    version=TelegramClient.__version__,
    description="Python3 Telegram's client implementation with full access to its API",
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/LonamiWebs/Telethon',
    download_url='https://github.com/LonamiWebs/Telethon/releases',

    # Author details
    author='Lonami Exo',
    author_email='totufals@hotmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Communications :: Chat',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    # What does your project relate to?
    keywords='Telegram API chat client MTProto',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=[
        'telethon_generator', 'telethon_tests', 'run_tests.py',
        'try_telethon.py'
    ]),

    # List run-time dependencies here. These will be installed by pip when
    # your project is installed.
    install_requires=['pyaes'],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'gen_tl = tl_generator:clean_and_generate',
        ],
    })



from datetime import datetime, timedelta


class MTProtoRequest:
    def __init__(self):
        self.sent = False

        self.msg_id = 0  # Long
        self.sequence = 0

        self.dirty = False
        self.send_time = None
        self.confirm_received = False

        # These should be overrode
        self.constructor_id = 0
        self.confirmed = False
        self.responded = False

    # These should not be overrode
    def on_send_success(self):
        self.send_time = datetime.now()
        self.sent = True

    def on_confirm(self):
        self.confirm_received = True

    def need_resend(self):
        return self.dirty or (
            self.confirmed and not self.confirm_received and
            datetime.now() - self.send_time > timedelta(seconds=3))

    # These should be overrode
    def on_send(self, writer):
        pass

    def on_response(self, reader):
        pass

    def on_exception(self, exception):
        pass



from ...tl.mtproto_request import MTProtoRequest


class InputEncryptedFileUploaded(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    inputEncryptedFileUploaded#64bd0306 id:long parts:int md5_checksum:string key_fingerprint:int = InputEncryptedFile"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x64bd0306

    def __init__(self, id, parts, md5_checksum, key_fingerprint):
        """
        :param id: Telegram type: «long».
        :param parts: Telegram type: «int».
        :param md5_checksum: Telegram type: «string».
        :param key_fingerprint: Telegram type: «int».

        Constructor for InputEncryptedFile: Instance of either InputEncryptedFileEmpty, InputEncryptedFileUploaded, InputEncryptedFile, InputEncryptedFileBigUploaded.
        """
        super().__init__()

        self.id = id
        self.parts = parts
        self.md5_checksum = md5_checksum
        self.key_fingerprint = key_fingerprint

    def on_send(self, writer):
        writer.write_int(InputEncryptedFileUploaded.constructor_id, signed=False)
        writer.write_long(self.id)
        writer.write_int(self.parts)
        writer.tgwrite_string(self.md5_checksum)
        writer.write_int(self.key_fingerprint)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InputEncryptedFileUploaded(None, None, None, None)

    def on_response(self, reader):
        self.id = reader.read_long()
        self.parts = reader.read_int()
        self.md5_checksum = reader.tgread_string()
        self.key_fingerprint = reader.read_int()

    def __repr__(self):
        return 'inputEncryptedFileUploaded#64bd0306 id:long parts:int md5_checksum:string key_fingerprint:int = InputEncryptedFile'

    def __str__(self):
        return '(inputEncryptedFileUploaded (ID: 0x64bd0306) = (id={}, parts={}, md5_checksum={}, key_fingerprint={}))'.format(str(self.id), str(self.parts), str(self.md5_checksum), str(self.key_fingerprint))



from ...tl.mtproto_request import MTProtoRequest


class Channel(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    channel#a14dca52 flags:# creator:flags.0?true kicked:flags.1?true left:flags.2?true editor:flags.3?true moderator:flags.4?true broadcast:flags.5?true verified:flags.7?true megagroup:flags.8?true restricted:flags.9?true democracy:flags.10?true signatures:flags.11?true min:flags.12?true id:int access_hash:flags.13?long title:string username:flags.6?string photo:ChatPhoto date:date version:int restriction_reason:flags.9?string = Chat"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xa14dca52

    def __init__(self, id, title, photo, date, version, creator=None, kicked=None, left=None, editor=None, moderator=None, broadcast=None, verified=None, megagroup=None, restricted=None, democracy=None, signatures=None, min=None, access_hash=None, username=None, restriction_reason=None):
        """
        :param creator: Telegram type: «true».
        :param kicked: Telegram type: «true».
        :param left: Telegram type: «true».
        :param editor: Telegram type: «true».
        :param moderator: Telegram type: «true».
        :param broadcast: Telegram type: «true».
        :param verified: Telegram type: «true».
        :param megagroup: Telegram type: «true».
        :param restricted: Telegram type: «true».
        :param democracy: Telegram type: «true».
        :param signatures: Telegram type: «true».
        :param min: Telegram type: «true».
        :param id: Telegram type: «int».
        :param access_hash: Telegram type: «long».
        :param title: Telegram type: «string».
        :param username: Telegram type: «string».
        :param photo: Telegram type: «ChatPhoto».
        :param date: Telegram type: «date».
        :param version: Telegram type: «int».
        :param restriction_reason: Telegram type: «string».

        Constructor for Chat: Instance of either ChatEmpty, Chat, ChatForbidden, Channel, ChannelForbidden.
        """
        super().__init__()

        self.creator = creator
        self.kicked = kicked
        self.left = left
        self.editor = editor
        self.moderator = moderator
        self.broadcast = broadcast
        self.verified = verified
        self.megagroup = megagroup
        self.restricted = restricted
        self.democracy = democracy
        self.signatures = signatures
        self.min = min
        self.id = id
        self.access_hash = access_hash
        self.title = title
        self.username = username
        self.photo = photo
        self.date = date
        self.version = version
        self.restriction_reason = restriction_reason

    def on_send(self, writer):
        writer.write_int(Channel.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.creator else 0
        flags |= (1 << 1) if self.kicked else 0
        flags |= (1 << 2) if self.left else 0
        flags |= (1 << 3) if self.editor else 0
        flags |= (1 << 4) if self.moderator else 0
        flags |= (1 << 5) if self.broadcast else 0
        flags |= (1 << 7) if self.verified else 0
        flags |= (1 << 8) if self.megagroup else 0
        flags |= (1 << 9) if self.restricted else 0
        flags |= (1 << 10) if self.democracy else 0
        flags |= (1 << 11) if self.signatures else 0
        flags |= (1 << 12) if self.min else 0
        flags |= (1 << 13) if self.access_hash else 0
        flags |= (1 << 6) if self.username else 0
        flags |= (1 << 9) if self.restriction_reason else 0
        writer.write_int(flags)

        writer.write_int(self.id)
        if self.access_hash:
            writer.write_long(self.access_hash)

        writer.tgwrite_string(self.title)
        if self.username:
            writer.tgwrite_string(self.username)

        self.photo.on_send(writer)
        writer.tgwrite_date(self.date)
        writer.write_int(self.version)
        if self.restriction_reason:
            writer.tgwrite_string(self.restriction_reason)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return Channel(None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 0)) != 0:
            self.creator = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 1)) != 0:
            self.kicked = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 2)) != 0:
            self.left = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 3)) != 0:
            self.editor = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 4)) != 0:
            self.moderator = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 5)) != 0:
            self.broadcast = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 7)) != 0:
            self.verified = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 8)) != 0:
            self.megagroup = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 9)) != 0:
            self.restricted = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 10)) != 0:
            self.democracy = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 11)) != 0:
            self.signatures = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 12)) != 0:
            self.min = True  # Arbitrary not-None value, no need to read since it is a flag

        self.id = reader.read_int()
        if (flags & (1 << 13)) != 0:
            self.access_hash = reader.read_long()

        self.title = reader.tgread_string()
        if (flags & (1 << 6)) != 0:
            self.username = reader.tgread_string()

        self.photo = reader.tgread_object()
        self.date = reader.tgread_date()
        self.version = reader.read_int()
        if (flags & (1 << 9)) != 0:
            self.restriction_reason = reader.tgread_string()

    def __repr__(self):
        return 'channel#a14dca52 flags:# creator:flags.0?true kicked:flags.1?true left:flags.2?true editor:flags.3?true moderator:flags.4?true broadcast:flags.5?true verified:flags.7?true megagroup:flags.8?true restricted:flags.9?true democracy:flags.10?true signatures:flags.11?true min:flags.12?true id:int access_hash:flags.13?long title:string username:flags.6?string photo:ChatPhoto date:date version:int restriction_reason:flags.9?string = Chat'

    def __str__(self):
        return '(channel (ID: 0xa14dca52) = (creator={}, kicked={}, left={}, editor={}, moderator={}, broadcast={}, verified={}, megagroup={}, restricted={}, democracy={}, signatures={}, min={}, id={}, access_hash={}, title={}, username={}, photo={}, date={}, version={}, restriction_reason={}))'.format(str(self.creator), str(self.kicked), str(self.left), str(self.editor), str(self.moderator), str(self.broadcast), str(self.verified), str(self.megagroup), str(self.restricted), str(self.democracy), str(self.signatures), str(self.min), str(self.id), str(self.access_hash), str(self.title), str(self.username), str(self.photo), str(self.date), str(self.version), str(self.restriction_reason))



import json
import os
import pickle
import random
import time
from threading import Lock
from base64 import b64encode, b64decode
from os.path import isfile as file_exists

from .. import helpers as utils


class Session:
    def __init__(self, session_user_id):
        self.session_user_id = session_user_id
        self.server_address = '91.108.56.165'
        self.port = 443
        self.auth_key = None
        self.id = utils.generate_random_long(signed=False)
        self.sequence = 0
        self.salt = 0  # Unsigned long
        self.time_offset = 0
        self.last_message_id = 0  # Long
        # TODO Remove this now unused members, left so unpickling is happy
        self.user = None

    def save(self):
        """Saves the current session object as session_user_id.session"""
        if self.session_user_id:
            with open('{}.session'.format(self.session_user_id), 'wb') as file:
                pickle.dump(self, file)

    def delete(self):
        """Deletes the current session file"""
        try:
            os.remove('{}.session'.format(self.session_user_id))
            return True
        except OSError:
            return False

    @staticmethod
    def try_load_or_create_new(session_user_id):
        """Loads a saved session_user_id session, or creates a new one if none existed before.
           If the given session_user_id is None, we assume that it is for testing purposes"""
        if session_user_id is None:
            return Session(None)
        else:
            path = '{}.session'.format(session_user_id)

            if file_exists(path):
                with open(path, 'rb') as file:
                    return pickle.load(file)
            else:
                return Session(session_user_id)

    def generate_sequence(self, confirmed):
        """Ported from JsonSession.generate_sequence"""
        with Lock():
            if confirmed:
                result = self.sequence * 2 + 1
                self.sequence += 1
                return result
            else:
                return self.sequence * 2

    def get_new_msg_id(self):
        """Generates a new message ID based on the current time (in ms) since epoch"""
        # Refer to mtproto_plain_sender.py for the original method, this is a simple copy
        ms_time = int(time.time() * 1000)
        new_msg_id = (((ms_time // 1000 + self.time_offset) << 32)
                      |  # "must approximately equal unix time*2^32"
                      ((ms_time % 1000) << 22)
                      |  # "approximate moment in time the message was created"
                      random.randint(0, 524288)
                      << 2)  # "message identifiers are divisible by 4"

        if self.last_message_id >= new_msg_id:
            new_msg_id = self.last_message_id + 4

        self.last_message_id = new_msg_id
        return new_msg_id

    def update_time_offset(self, correct_msg_id):
        """Updates the time offset based on a known correct message ID"""
        now = int(time.time())
        correct = correct_msg_id >> 32
        self.time_offset = correct - now


# Until migration is complete, we need the original 'Session' class
# for Pickle to keep working. TODO Replace 'Session' by 'JsonSession' by v1.0
class JsonSession:
    """This session contains the required information to login into your
       Telegram account. NEVER give the saved JSON file to anyone, since
       they would gain instant access to all your messages and contacts.

       If you think the session has been compromised, close all the sessions
       through an official Telegram client to revoke the authorization.
    """
    def __init__(self, session_user_id):
        self.session_user_id = session_user_id
        self.server_address = '91.108.56.165'
        self.port = 443
        self.auth_key = None
        self.id = utils.generate_random_long(signed=False)
        self._sequence = 0
        self.salt = 0  # Unsigned long
        self.time_offset = 0
        self.last_message_id = 0  # Long

        # Cross-thread safety
        self._lock = Lock()

    def save(self):
        """Saves the current session object as session_user_id.session"""
        if self.session_user_id:
            with open('{}.session'.format(self.session_user_id), 'w') as file:
                json.dump({
                    'id': self.id,
                    'port': self.port,
                    'salt': self.salt,
                    'sequence': self._sequence,
                    'time_offset': self.time_offset,
                    'server_address': self.server_address,
                    'auth_key_data':
                        b64encode(self.auth_key.key).decode('ascii')
                }, file)

    def delete(self):
        """Deletes the current session file"""
        try:
            os.remove('{}.session'.format(self.session_user_id))
            return True
        except OSError:
            return False

    @staticmethod
    def try_load_or_create_new(session_user_id):
        """Loads a saved session_user_id.session or creates a new one.
           If session_user_id=None, later .save()'s will have no effect.
        """
        if session_user_id is None:
            return JsonSession(None)
        else:
            path = '{}.session'.format(session_user_id)
            result = JsonSession(session_user_id)
            if not file_exists(path):
                return result

            try:
                with open(path, 'r') as file:
                    data = json.load(file)
                    result.id = data['id']
                    result.port = data['port']
                    result.salt = data['salt']
                    result._sequence = data['sequence']
                    result.time_offset = data['time_offset']
                    result.server_address = data['server_address']

                    # FIXME We need to import the AuthKey here or otherwise
                    # we get cyclic dependencies.
                    from ..crypto import AuthKey
                    key = b64decode(data['auth_key_data'])
                    result.auth_key = AuthKey(data=key)

            except (json.decoder.JSONDecodeError, UnicodeDecodeError):
                # TODO Backwards-compatibility code
                old = Session.try_load_or_create_new(session_user_id)
                result.id = old.id
                result.port = old.port
                result.salt = old.salt
                result._sequence = old.sequence
                result.time_offset = old.time_offset
                result.server_address = old.server_address
                result.auth_key = old.auth_key
                result.save()

            return result

    def generate_sequence(self, confirmed):
        """Thread safe method to generates the next sequence number,
           based on whether it was confirmed yet or not.

           Note that if confirmed=True, the sequence number
           will be increased by one too
        """
        with self._lock:
            if confirmed:
                result = self._sequence * 2 + 1
                self._sequence += 1
                return result
            else:
                return self._sequence * 2

    def get_new_msg_id(self):
        """Generates a new unique message ID based on the current
           time (in ms) since epoch"""
        # Refer to mtproto_plain_sender.py for the original method,
        ms_time = int(time.time() * 1000)
        new_msg_id = (((ms_time // 1000 + self.time_offset) << 32)
                      |  # "must approximately equal unix time*2^32"
                      ((ms_time % 1000) << 22)
                      |  # "approximate moment in time the message was created"
                      random.randint(0, 524288)
                      << 2)  # "message identifiers are divisible by 4"

        if self.last_message_id >= new_msg_id:
            new_msg_id = self.last_message_id + 4

        self.last_message_id = new_msg_id
        return new_msg_id

    def update_time_offset(self, correct_msg_id):
        """Updates the time offset based on a known correct message ID"""
        now = int(time.time())
        correct = correct_msg_id >> 32
        self.time_offset = correct - now



from ...tl.mtproto_request import MTProtoRequest


class UpdateChannel(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateChannel#b6d45656 channel_id:int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xb6d45656

    def __init__(self, channel_id):
        """
        :param channel_id: Telegram type: «int».

        Constructor for Update: Instance of either UpdateNewMessage, UpdateMessageID, UpdateDeleteMessages, UpdateUserTyping, UpdateChatUserTyping, UpdateChatParticipants, UpdateUserStatus, UpdateUserName, UpdateUserPhoto, UpdateContactRegistered, UpdateContactLink, UpdateNewEncryptedMessage, UpdateEncryptedChatTyping, UpdateEncryption, UpdateEncryptedMessagesRead, UpdateChatParticipantAdd, UpdateChatParticipantDelete, UpdateDcOptions, UpdateUserBlocked, UpdateNotifySettings, UpdateServiceNotification, UpdatePrivacy, UpdateUserPhone, UpdateReadHistoryInbox, UpdateReadHistoryOutbox, UpdateWebPage, UpdateReadMessagesContents, UpdateChannelTooLong, UpdateChannel, UpdateNewChannelMessage, UpdateReadChannelInbox, UpdateDeleteChannelMessages, UpdateChannelMessageViews, UpdateChatAdmins, UpdateChatParticipantAdmin, UpdateNewStickerSet, UpdateStickerSetsOrder, UpdateStickerSets, UpdateSavedGifs, UpdateBotInlineQuery, UpdateBotInlineSend, UpdateEditChannelMessage, UpdateChannelPinnedMessage, UpdateBotCallbackQuery, UpdateEditMessage, UpdateInlineBotCallbackQuery, UpdateReadChannelOutbox, UpdateDraftMessage, UpdateReadFeaturedStickers, UpdateRecentStickers, UpdateConfig, UpdatePtsChanged, UpdateChannelWebPage, UpdateDialogPinned, UpdatePinnedDialogs, UpdateBotWebhookJSON, UpdateBotWebhookJSONQuery, UpdateBotShippingQuery, UpdateBotPrecheckoutQuery, UpdatePhoneCall.
        """
        super().__init__()

        self.channel_id = channel_id

    def on_send(self, writer):
        writer.write_int(UpdateChannel.constructor_id, signed=False)
        writer.write_int(self.channel_id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateChannel(None)

    def on_response(self, reader):
        self.channel_id = reader.read_int()

    def __repr__(self):
        return 'updateChannel#b6d45656 channel_id:int = Update'

    def __str__(self):
        return '(updateChannel (ID: 0xb6d45656) = (channel_id={}))'.format(str(self.channel_id))



from ...tl.mtproto_request import MTProtoRequest


class InputMediaUploadedPhoto(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    inputMediaUploadedPhoto#630c9af1 flags:# file:InputFile caption:string stickers:flags.0?Vector<InputDocument> = InputMedia"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x630c9af1

    def __init__(self, file, caption, stickers=None):
        """
        :param file: Telegram type: «InputFile».
        :param caption: Telegram type: «string».
        :param stickers: Telegram type: «InputDocument». Must be a list.

        Constructor for InputMedia: Instance of either InputMediaEmpty, InputMediaUploadedPhoto, InputMediaPhoto, InputMediaGeoPoint, InputMediaContact, InputMediaUploadedDocument, InputMediaUploadedThumbDocument, InputMediaDocument, InputMediaVenue, InputMediaGifExternal, InputMediaPhotoExternal, InputMediaDocumentExternal, InputMediaGame, InputMediaInvoice.
        """
        super().__init__()

        self.file = file
        self.caption = caption
        self.stickers = stickers

    def on_send(self, writer):
        writer.write_int(InputMediaUploadedPhoto.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.stickers else 0
        writer.write_int(flags)

        self.file.on_send(writer)
        writer.tgwrite_string(self.caption)
        if self.stickers:
            writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
            writer.write_int(len(self.stickers))
            for stickers_item in self.stickers:
                if stickers_item:
                    stickers_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InputMediaUploadedPhoto(None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        self.file = reader.tgread_object()
        self.caption = reader.tgread_string()
        if (flags & (1 << 0)) != 0:
            reader.read_int()  # Vector's constructor ID
            self.stickers = []  # Initialize an empty list
            stickers_len = reader.read_int()
            for _ in range(stickers_len):
                stickers_item = reader.tgread_object()
                self.stickers.append(stickers_item)

    def __repr__(self):
        return 'inputMediaUploadedPhoto#630c9af1 flags:# file:InputFile caption:string stickers:flags.0?Vector<InputDocument> = InputMedia'

    def __str__(self):
        return '(inputMediaUploadedPhoto (ID: 0x630c9af1) = (file={}, caption={}, stickers={}))'.format(str(self.file), str(self.caption), None if not self.stickers else [str(_) for _ in self.stickers])
